# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.utils.bridge.vbridge import wts_bridge
from bayeslibs.config import const


class ApolloWakeUper:
    """
    语音唤醒模块封装类
    """
    def __init__(self):
        pass

    @staticmethod
    def open(text):
        return open_wakeup(text)

    @staticmethod
    def close():
        return close_wakeup()

    @staticmethod
    def status():
        return get_wakeup_status()


def open_wakeup(text=''):
    """
    开启机器人语音唤醒
    :param:
    :return:result
    :example:
        result = open_wakeup('你好，同学')
        ------
        result:{
            'status':0,
            'msg':'start wts success'
        }
    """
    result = wts_bridge(req_type=const.TYPE_START, text=text)
    return result


def close_wakeup():
    """
    关闭机器人语音唤醒
    :param
    :return:result
    :example:
        result = close_wakeup()
        ------
        result:{
            'status':0,
            'msg':'stop success'
        }
    """
    result = wts_bridge(req_type=const.TYPE_STOP)
    return result


def get_wakeup_status():
    """
    查询机器人语音唤醒状态
    :param
    :return:result
    :example:
        result = get_wakeup_status()
        ------
        result:{
            'status':0,
            'msg':'voice is playing',
            'angle':120
        }
    """
    result = wts_bridge(req_type=const.TYPE_QUERY)
    return result



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import open_distance_detector, get_distance_detected
from bayeslibs.vision import close_distance_detector

# 程序运行时间
RUN_TIME = 10


def center_point_distance_detect_sample():
    """
       利用APOLLO机器人，检测特定图像的中央点到深度摄像头之间的距离
    """
    # 限定深度图像获取范围
    pos = {
        'left': 270,
        'top': 190,
        'width': 50,
        'height': 50
    }
    open_res = open_distance_detector(pos)  # 打开距离检测功能
    if open_res['status'] == 0:
        print('>>> 图像中心点距离检测功能打开成功 <<<')
        start_time = datetime.datetime.now()
        cur_time = datetime.datetime.now()
        while (cur_time - start_time).seconds < RUN_TIME:
            # 获取距离检测数据，数据格式参考get_distance_detected函数说明
            dist_detected = get_distance_detected()
            print('-----------------------')
            # status=0表示正确检测到距离，条件判断通过后进行距离数据解析
            if dist_detected['status'] == 0:
                print('distance: {}m'.format(dist_detected['data']['dist']))
            else:
                print('没有检测到距离信息，请调整至合适位置')
            cur_time = datetime.datetime.now()
        print('-----------------------')
        print('{}s运行时间已到，退出程序'.format(RUN_TIME))
        close_distance_detector()
        return True
    else:
        return False


if __name__ == '__main__':
    '''
    Python以模块运行方式启动入口
    '''
    center_point_distance_detect_sample()



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from abc import abstractmethod

from bayeslibs.motion import robot_move_forward, get_robot_move_status, robot_move_back


class ApolloMoveBase:
    def __init__(self):
        pass

    @staticmethod
    def get_move_status():
        try:
            print('------------------------------------')
            print('运动中.....')
            move_stat = get_robot_move_status()
            while move_stat['status'] != 0:
                if move_stat['status'] == 505:
                    print('运动程序不存在')
                    return False
                move_stat = get_robot_move_status()
            print('------------------------------------')
            print('运动完成.....')
            print('------------------------------------')
            return True
        except Exception as err:
            print('发生未知错误:{}'.format(err))
            return False

    @abstractmethod
    def move(self, distance):
        pass


class ApolloMoveForward(ApolloMoveBase):
    def __init__(self):
        super().__init__()

    def move(self, distance):
        robot_move_forward(distance)
        return self.get_move_status()


class ApolloMoveBack(ApolloMoveBase):
    def __init__(self):
        super().__init__()

    def move(self, distance):
        robot_move_back(distance)
        return self.get_move_status()


def apollo_move_sample(motion_type, para):
    """
    自定义距离，让小车实现直线运动
    :param motion_type: 向前走，向后走
    :param para: 1.5
    :return:
    """
    if motion_type == '向前走':
        # 初始化Apollo运动模块实例
        apollo_move = ApolloMoveForward()
        print('------------------------------------')
        print('发送向前走{}m的运动指令'.format(para))
        apollo_move.move(para)  # 向前走**m
        return True
    elif motion_type == '向后走':
        # 初始化Apollo运动模块实例
        apollo_move = ApolloMoveBack()
        print('------------------------------------')
        print('发送向后走{}m的运动指令'.format(para))
        apollo_move.move(para)  # 向前走**m
        return True
    else:
        print('------------------------------------')
        print('输入参数有误，请重新输入')
        return False


if __name__ == '__main__':
    count = 0
    # 程序运行10次，次数可修改
    # 运动指令：向前走，向后走
    # 运动参数：1,1.4,2,3,不要超过10m
    while count < 10:
        motion_type_ = input('请输入运动指令:')
        para_ = float(input('请输入运动参数:'))
        apollo_move_sample(motion_type_, para_)
        count += 1



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.utils.bridge.vbridge import chat_bridge
from bayeslibs.config import const


class ApolloChatter:
    """
    语音闲聊模块封装类
    """
    def __init__(self):
        pass

    @staticmethod
    def start(text):
        return start_small_talk(text)

    @staticmethod
    def stop():
        return stop_small_talk()


def start_small_talk(text):
    """
    开启机器人语音闲聊
    :param:
    :return:result
    :example:
        result = start_small_talk('你好帅')
        ------
        result:{
            'status':0,
            'msg':'success',
            'answer':'你也好帅了'
        }
    """
    result = chat_bridge(req_type=const.TYPE_START, text=text)
    return result


def stop_small_talk():
    """
    关闭机器人语音闲聊
    :param
    :return:result
    :example:
        result = stop_small_talk()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = chat_bridge(req_type=const.TYPE_STOP)
    return result



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.voice import start_small_talk, start_speak, get_speak_status


def voice_play(text):
    """
       输入任意内容，让APOLLO机器人将其播放出来，实现语音合成功能
    """
    start_res = start_speak(text)
    if start_res and start_res['status'] == 0:
        res = get_speak_status()
        print('小贝语音合成中.......')
        print('------------------------------------')
        while res['status'] == 0:
            res = get_speak_status()
        return True
    else:
        return False


def apollo_chat_sample(question):
    """
    语音输入自己想说的话，观察APOLLO的自然语言处理结果
    """
    result = start_small_talk(question)
    if result['status'] == 0:
        answer = result['answer']
        print('小贝的回答:{}'.format(answer))
    else:
        print('小贝还小，还有很多需要学习')
        answer = '小贝还小，还有很多需要学习'
    print('------------------------------------')
    return answer


if __name__ == '__main__':
    count_ = 0
    # 循环聊天10次，次数可修改
    while count_ < 10:
        question_ = input('请输入您的问题:')
        print('------------------------------------')
        answer_ = apollo_chat_sample(question=question_)
        voice_play(answer_)
        count_ += 1



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import open_face_recognizer, get_faces_recognized, close_face_recognizer

# 程序运行时间
RUN_TIME = 30


def multi_face_recog_sample():
    """
       多人人脸识别
    """
    open_res = open_face_recognizer()  # 打开人脸识别功能
    if open_res['status'] == 0:
        print('>>> 多人人脸识别功能打开成功 <<<')
        start_time = datetime.datetime.now()
        cur_time = datetime.datetime.now()
        while (cur_time - start_time).seconds < RUN_TIME:
            # 获取人脸识别数据，数据格式参考get_faces_recognized函数说明
            faces_recognized = get_faces_recognized()
            print('-----------------------')
            # status=0表示正确检测到人脸，条件判断通过后进行人脸数据解析
            if faces_recognized['status'] == 0:
                print('人脸识别成功，人脸数目：{}'.format(len(faces_recognized['data'])))
                for face in faces_recognized['data']:
                    print('人脸名称:', face['face_name'], end=', ')
                    print('人脸得分:', face['score'])
            else:
                print('没有识别到人脸信息，请调整至合适位置')
            cur_time = datetime.datetime.now()
        print('-----------------------')
        print('{}s运行时间已到，退出程序'.format(RUN_TIME))
        close_face_recognizer()
        return True
    else:
        return False


if __name__ == '__main__':
    multi_face_recog_sample()



"""
@project:medical_robot_backend
@language:python3
@create:2019/4/26
@author:qianyang@aibayes.com
@description:none
"""
import logging
import inspect
import os
from logging.handlers import RotatingFileHandler
from .setting import LOG_DICT


class Logger:
    def __init__(self):
        pass
    
    @staticmethod
    def __create_logger(module):
        if module not in LOG_DICT:
            module = 'default'
        log_name = LOG_DICT[module]['log_name']
        msg_fmt = '[%(levelname)s][%(asctime)s]%(message)s'
        date_fmt = '%Y-%m-%d %H:%M:%S'
        formatter = logging.Formatter(fmt=msg_fmt, datefmt=date_fmt)
        _rt_handler = RotatingFileHandler(log_name, maxBytes=20 * 1024 * 1024, backupCount=5)
        _rt_handler.setFormatter(formatter)
        logger_ = logging.getLogger(module)
        if len(logger_.handlers) < 1:
            logger_.addHandler(_rt_handler)
        logger_.setLevel(LOG_DICT[module]['log_level'])
        return logger_

    @staticmethod
    def __log_message(file_path, line_no, author, message):
        return '[{0}][line:{1}][{2}]{3}'.format(file_path, line_no, author, message)

    @staticmethod
    def __log_writer(logger_, level, msg):
        if level is 'critical':
            logger_.critical(msg)
        elif level is 'error':
            logger_.error(msg)
        elif level is 'warning':
            logger_.warning(msg)
        elif level is 'info':
            logger_.info(msg)
        else:
            logger_.debug(msg)

    @staticmethod
    def __module_path(call_stack):
        file_path = os.path.basename(call_stack)
        dir_path = os.path.basename(os.path.dirname(call_stack))
        dir_dir_path = os.path.basename(os.path.dirname(os.path.dirname(call_stack)))
        if dir_path and dir_path is not file_path:
            file_path = '/{}/{}'.format(dir_path, file_path)
        if dir_dir_path and dir_dir_path is not dir_path:
            file_path = '.../{}{}'.format(dir_dir_path, file_path)
        return file_path
    
    @staticmethod
    def __log_msg(module='default'):
        try:
            logger_ = Logger.__create_logger(module)
        except KeyError:
            logger_ = Logger.__create_logger('other')
            logger_.error('ModuleInputError!')
        return logger_

    @staticmethod
    def debug(message, author='bayes', module='default'):
        logger_ = Logger.__log_msg(module)
        call_stack = inspect.stack()[1]
        file_path = Logger.__module_path(call_stack[1])
        line_no = call_stack[2]
        msg = Logger.__log_message(file_path, line_no, author, message)
        logger_.debug(msg)
        
    @staticmethod
    def info(message, author='bayes', module='default'):
        logger_ = Logger.__log_msg(module)
        call_stack = inspect.stack()[1]
        file_path = Logger.__module_path(call_stack[1])
        line_no = call_stack[2]
        msg = Logger.__log_message(file_path, line_no, author, message)
        logger_.info(msg)

    @staticmethod
    def warning(message, author='bayes', module='default'):
        logger_ = Logger.__log_msg(module)
        call_stack = inspect.stack()[1]
        file_path = Logger.__module_path(call_stack[1])
        line_no = call_stack[2]
        msg = Logger.__log_message(file_path, line_no, author, message)
        logger_.warning(msg)

    @staticmethod
    def error(message, author='bayes', module='default'):
        logger_ = Logger.__log_msg(module)
        call_stack = inspect.stack()[1]
        file_path = Logger.__module_path(call_stack[1])
        line_no = call_stack[2]
        msg = Logger.__log_message(file_path, line_no, author, message)
        logger_.error(msg)

    @staticmethod
    def critical(message, author='bayes', module='default'):
        logger_ = Logger.__log_msg(module)
        call_stack = inspect.stack()[1]
        file_path = Logger.__module_path(call_stack[1])
        line_no = call_stack[2]
        msg = Logger.__log_message(file_path, line_no, author, message)
        logger_.critical(msg)



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.motion import start_auto_nav, get_auto_nav_status, stop_auto_nav
from bayeslibs.vision import open_face_detector, close_face_detector, get_faces_detected
from bayeslibs.vision import open_face_recognizer, close_face_recognizer, get_faces_recognized
from bayeslibs.vision import open_handpose_recognizer, close_handpose_recognizer
from bayeslibs.vision import get_handposes_recognized
from bayeslibs.config import get_slam_pos, add_slam_pos
import time

FOOD_ROOM = 'p_food'
DINING_TABLE = 'p_12table'
ORIGIN_PLACE = 'p_origin'
DESTINATION_POS_MAP = {'p_food': {'x': 4.668, 'y': 7.275},
                       'p_12table': {'x': 0.312, 'y': 6.148}, 'p_origin': {'x': -0.337, 'y': 3.158}}
NAV_TIME = 60 * 20

FACE_DETECT_TIME = 30
FACE_PROBABILITY = 0.9
HANDPOSE_LIST = ['One', 'Two', 'Ok']

HANDPOSE_RUN_TIME = 30
HANDPOSE_LOOP = 2

FACE_RECOG_TIME = 30
VIP_MAP = {'chenwei': '陈伟', 'shuqing': '舒庆', 'qianyang': '钱扬', 'yewei': '叶伟'}


def update_nav_pos():
    """
    更新室内导航地图位置信息
    :return: True
    """
    for item, pos in DESTINATION_POS_MAP.items():
        add_slam_pos(item, pos['x'], pos['y'])
    return True


class ApolloAGV:
    def __init__(self):
        self.handpose = list()

    @staticmethod
    def voice_notice(text):
        """
           输入提示信息，让APOLLO机器人将其播放出来
        """
        from bayeslibs.voice import start_speak, get_speak_status
        start_res = start_speak(text)
        if start_res and start_res['status'] == 0:
            print('>>> 小贝语音合成功能打开成功 <<<')
            s_res = get_speak_status()
            print('------------------------------------')
            print('小贝语音合成中.......')
            while s_res['status'] == 0:
                s_res = get_speak_status()
            time.sleep(0.2)
            print('------------------------------------')
            print('>>> 小贝语音合成结束 <<<')
            print('------------------------------------')
            return True
        else:
            return False

    def handpose_recog(self):
        """
           捕捉摄像头前的OK手势
        """
        open_res = open_handpose_recognizer()  # 打开年龄性别检测功能
        if open_res['status'] == 0:
            print('>>> 手势识别功能打开成功 <<<')
            start_time = datetime.datetime.now()
            cur_time = datetime.datetime.now()
            while (cur_time - start_time).seconds < HANDPOSE_RUN_TIME:
                # 获取手势识别数据，数据格式参考get_handposes_recognized函数说明
                handpose_recognized = get_handposes_recognized()
                # status=0表示正确检测到手势，条件判断通过后进行手势数据解析
                if handpose_recognized['status'] == 0:
                    print('手势识别成功，手势识别数目：{}'.format(len(handpose_recognized['data'])))
                    for handpose in handpose_recognized['data']:
                        if handpose['classname'] not in HANDPOSE_LIST:
                            print('-----------------------')
                            print('手势不对，请调整至OK')
                            continue
                        if len(self.handpose) == 0 and handpose['classname'] == 'Ok':
                            print('-----------------------')
                            print('手势OK识别正确')
                            self.handpose.append('OK')
                            print('-----------------------')
                            self.voice_notice('菜品已经成功放置，谢谢')
                            close_handpose_recognizer()
                            return True
                cur_time = datetime.datetime.now()
            print('-----------------------')
            print('{}s运行时间已到，退出程序'.format(HANDPOSE_RUN_TIME))
            close_handpose_recognizer()
            return False
        else:
            return False

    def face_detect(self):
        """
           多人人脸检测
        """
        open_res = open_face_detector()  # 打开人脸检测功能
        if open_res['status'] == 0:
            print('>>> 人脸检测功能打开成功 <<<')
            start_time = datetime.datetime.now()
            cur_time = datetime.datetime.now()
            while (cur_time - start_time).seconds < FACE_DETECT_TIME:
                # 获取人脸检测数据，数据格式参考get_faces_detected函数说明
                faces_detected = get_faces_detected()
                print('-----------------------')
                # status=0表示正确检测到人脸，条件判断通过后进行人脸数据解析
                if faces_detected['status'] == 0:
                    print('人脸检测成功，人脸数目：{}'.format(len(faces_detected['data'])))
                    for face in faces_detected['data']:
                        print('目标名称:', face['object_name'], end=', ')
                        print('人脸概率:', face['probability'])
                        if face['probability'] > FACE_PROBABILITY:
                            self.voice_notice('您好，请将12号桌的菜品放到面板上，菜品放置完毕后， 请做OK的手势')
                            close_face_detector()
                            return True
                else:
                    print('没有检测到人脸信息，请调整至合适位置')
                cur_time = datetime.datetime.now()
            print('-----------------------')
            print('{}s运行时间已到，退出程序'.format(FACE_DETECT_TIME))
            close_face_detector()
            return False
        else:
            return False

    def vip_recog_notice(self):
        """
           人脸识别
        """
        open_res = open_face_recognizer()  # 打开人脸识别功能
        if open_res['status'] == 0:
            print('>>> 多人人脸识别功能打开成功 <<<')
            start_time = datetime.datetime.now()
            cur_time = datetime.datetime.now()
            while (cur_time - start_time).seconds < FACE_RECOG_TIME:
                # 获取人脸识别数据，数据格式参考get_faces_recognized函数说明
                faces_recognized = get_faces_recognized()
                print('-----------------------')
                # status=0表示正确检测到人脸，条件判断通过后进行人脸数据解析
                if faces_recognized['status'] == 0:
                    print('人脸识别成功，人脸数目：{}'.format(len(faces_recognized['data'])))
                    for face in faces_recognized['data']:
                        print('人脸名称:', face['face_name'], end=', ')
                        print('人脸得分:', face['score'])
                        if face['face_name'] in VIP_MAP.keys():
                            self.voice_notice('{}，请取走您的餐品'.format(VIP_MAP[face['face_name']]))
                            close_face_recognizer()
                            return True
                else:
                    print('没有识别到VIP信息，请调整至合适位置')
                cur_time = datetime.datetime.now()
            print('-----------------------')
            print('{}s运行时间已到，退出程序'.format(FACE_RECOG_TIME))
            close_face_recognizer()
            return False
        else:
            return False

    def get_medicine(self):
        try:
            f_res = self.face_detect()
            if not f_res:
                self.voice_notice('您好，请将12号桌的菜品放到面板上，菜品放置完毕后， 请做OK的手势')
            count = 1
            time.sleep(2)
            self.voice_notice('您好，请您做OK的手势')
            h_res = self.handpose_recog()
            self.handpose = list()
            while not h_res:
                if count > HANDPOSE_LOOP:
                    return False
                self.voice_notice('手势无法正确识别，请重新做OK的手势')
                print('------------------------------------')
                print('重新进行手势识别')
                count += 1
                h_res = self.handpose_recog()
            return True
        except Exception as err:
            print('ApolloAGV get_medicine error:{}'.format(err))
            return False

    @staticmethod
    def auto_guided_position(dest):
        if dest not in get_slam_pos():
            print('地点输入有误，请重新配置导航地点')
            return False
        stat_res = start_auto_nav(dest)
        if stat_res and stat_res['status'] == 0:
            print('>>> 小贝自动导航功能打开成功 <<<')
            start_time = datetime.datetime.now()
            cur_time = datetime.datetime.now()
            stat = get_auto_nav_status()
            print('------------------------------------')
            print('小贝导航中.......')
            # 60*20s如果还没有导航成功就取消导航
            while stat and stat['status'] != 0 and (cur_time - start_time).seconds <= NAV_TIME:
                stat = get_auto_nav_status()
                cur_time = datetime.datetime.now()
            if (cur_time - start_time).seconds > NAV_TIME:
                print('------------------------------------')
                print('小贝导航超时.......{}s运行时间已到，退出程序'.format(NAV_TIME))
                n_res = stop_auto_nav()
                if n_res['status'] == 0:
                    print('导航关闭成功')
                    return False
            print('------------------------------------')
            print('小贝导航成功.......')
            print('------------------------------------')
            return True
        else:
            print('网络故障，请检查网络状况是否良好')
            return False

    def back_home(self):
        o_nav_res = self.auto_guided_position(ORIGIN_PLACE)
        if not o_nav_res:
            print('------------------------------------')
            print('返回原点导航失败，请检查程序和小车配置，确认无误后重启程序')
            return False
        return True

    def auto_guided_vehicle(self):
        try:
            f_nav_res = self.auto_guided_position(FOOD_ROOM)
            if not f_nav_res:
                print('------------------------------------')
                print('取餐室导航失败，小车将返回原点')
                return self.back_home()
            md_res = self.get_medicine()
            if not md_res:
                print('------------------------------------')
                print('取药失败，小车将返回原点')
                return self.back_home()
            d_nav_res = self.auto_guided_position(DINING_TABLE)
            if not d_nav_res:
                print('------------------------------------')
                print('12号餐桌导航失败，小车将返回原点')
                return self.back_home()
            vip_res = self.vip_recog_notice()
            if not vip_res:
                print('------------------------------------')
                print('VIP用户无法识别，小车将返回原点')
            time.sleep(1)
            self.voice_notice('菜品已经成功送达，小贝要回去了，谢谢')
            return self.back_home()
        except Exception as err:
            print('ApolloAGV error:{}'.format(err))
            return False


if __name__ == '__main__':
    # 更新室内导航地图位置信息
    update_nav_pos()
    apollo_cruise = ApolloAGV()
    res = apollo_cruise.auto_guided_vehicle()
    if res:
        print('------------------------------------')
        print('程序运行成功')
    else:
        print('------------------------------------')
        print('程序异常退出')



"""
@project：medical_robot_backend
@language：Python3
@create：2019/5/18
@author：qianyang<qianyang@aibayes.com>
@description:none
"""


def object2dict(form):
    form_dict = dict()
    if isinstance(form, object):
        form_attr = [attr for attr in dir(form) if not attr.startswith('__') and not callable(getattr(form, attr))]
        for attr_name in form_attr:
            form_dict[attr_name] = getattr(form, attr_name)
    return form_dict



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import time
from bayeslibs.voice import open_asr, get_asr_status, start_speak, close_asr


def auto_speech_recognize_sample():
    """
    APOLLO机器人语音识别
    """
    res_asr = open_asr()
    if res_asr and res_asr['status'] == 0:
        print('>>> 小贝语音识别功能打开成功 <<<')
        stat = get_asr_status()
        while stat and stat['is_asr']:
            print('------------------------------------')
            stat = get_asr_status()
            print('小贝识别中.......')
        print('------------------------------------')
        if 'text' in stat and stat['text']:
            print('小贝识别成功:{}'.format(stat['text']))
            start_speak(stat['text'])
            print('------------------------------------')
            time.sleep(2)
        else:
            print('小贝识别失败')
            print('------------------------------------')
            start_speak('不好意思，小贝没有听清你的声音')
            time.sleep(2)
        print('语音播放中.......')
        close_asr()
        print('------------------------------------')
        print('运行结束，退出程序')
        return True
    else:
        return False


if __name__ == '__main__':
    auto_speech_recognize_sample()



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/6/21
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import base64
import json
from urllib.parse import urlencode
from urllib.request import urlopen, Request
import os


class BayesFaceRegister:
    __APP_ID = '14987471'
    __API_KEY = '5T5yB8G2OpFdCcVb1OK7ipap'
    __SECRET_KEY = 'juNdil9rcizpakslmUXSHRQzuo9h0T96'
    __GROUP_ID = 'bayes_test'
    __IMAGE_TYPE = 'BASE64'
    __TOKEN_URL = 'https://aip.baidubce.com/oauth/2.0/token'
    __USER_ADD_URL = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/user/add'
    __USER_UPDATE_URL = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/user/update'
    __USER_DELETE_URL = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/face/delete'
    __USER_GET_URL = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/user/get'

    def __init__(self):
        super().__init__()

    def token_acquire(self):
        params = {'grant_type': 'client_credentials',
                  'client_id': self.__API_KEY,
                  'client_secret': self.__SECRET_KEY}
        post_data = urlencode(params).encode('utf-8')
        req_ = Request(self.__TOKEN_URL, post_data)
        try:
            f_ = urlopen(req_, timeout=5)
            result_str_ = f_.read().decode('utf-8')
            res = json.loads(result_str_)
            f_.close()
            if 'access_token' in res:
                return res['access_token']
            else:
                raise Exception('access_token not in result')
        except Exception as err:
            print('BayesFaceRecognize token_acquire failed: {}'.format(err))
            return None

    @staticmethod
    def img2base64(img_src):
        try:
            with open(img_src, 'rb') as f:
                b64_data = base64.b64encode(f.read())
            return b64_data
        except Exception as err:
            print('img open failed:{}'.format(err))
            return None

    def http_request(self, request_url, params):
        access_token = self.token_acquire()
        if access_token:
            request_url = '{}?access_token={}'.format(request_url, access_token)
            request = Request(url=request_url, data=params)
            request.add_header('Content-Type', 'application/json')
            response = urlopen(request, timeout=5)
            content = response.read().decode('utf-8')
            response.close()
            if content:
                return json.loads(content)
            else:
                return None
        else:
            return None

    def register(self, img_src, user_id, group_id=__GROUP_ID):
        img_base64 = self.img2base64(img_src)
        if not img_base64:
            return None
        img_type = self.__IMAGE_TYPE
        request_url = self.__USER_ADD_URL
        data = {
            'image': img_base64,
            'image_type': img_type,
            'group_id': group_id,
            'user_id': user_id,
            'user_info': '{}\'s face'.format(user_id)
        }
        params = urlencode(data).encode('utf-8')
        return self.http_request(request_url, params)

    def update(self, img_src, user_id, group_id=__GROUP_ID):
        img_base64 = self.img2base64(img_src)
        if not img_base64:
            return None
        img_type = self.__IMAGE_TYPE
        request_url = self.__USER_UPDATE_URL
        data = {
            'image': img_base64,
            'image_type': img_type,
            'group_id': group_id,
            'user_id': user_id,
            'user_info': '{}\'s face'.format(user_id)
        }
        params = urlencode(data).encode('utf-8')
        return self.http_request(request_url, params)

    def delete_face(self, user_id, face_token, group_id=__GROUP_ID):
        request_url = self.__USER_DELETE_URL
        data = {
            'group_id': group_id,
            'user_id': user_id,
            'face_token': face_token
        }
        params = urlencode(data).encode('utf-8')
        return self.http_request(request_url, params)

    def get_face(self, user_id, group_id=__GROUP_ID):
        request_url = self.__USER_GET_URL
        data = {
            'group_id': group_id,
            'user_id': user_id
        }
        params = urlencode(data).encode('utf-8')
        return self.http_request(request_url, params)


def face_register(face_path):
    """
    多张人脸图片录入接口
    :param face_path: 多张人脸图片所在的文件夹路径
    :return:
    """
    try:
        if '\u202a' in face_path:
            face_path = face_path[1:]
        dir_files = os.listdir(face_path)
    except FileNotFoundError:
        print('input face path is wrong, please check your path')
        print('程序异常退出!')
        return False
    except OSError as err:
        print('input face path is wrong, please check your path, err:{}'.format(err))
        print('程序异常退出!')
        return False
    bayes_face = BayesFaceRegister()
    for filename in dir_files:
        file_path = '{}'.format(os.path.join(face_path, filename))
        if os.path.isdir(file_path):
            print('input face path is wrong, please check your path')
            continue
        if os.path.isfile(file_path):
            user_id = filename.split('.')[0]
            img_format = filename.split('.')[-1]
            if img_format not in ['png', 'jpg', 'bmp']:
                print('{} is not a supported image format'.format(filename))
                continue
            try:
                res = bayes_face.register(file_path, user_id=user_id)
                if res and res['error_code'] == 0:
                    print('{} register success'.format(user_id))
                elif res:
                    print('{} register failed:{}'.format(user_id, res['error_msg']))
                    continue
                else:
                    print('发生未知异常错误')
                    continue
            except Exception as err:
                print('{} register failed, error:{}'.format(user_id, err))
                print('程序异常退出!')
                return False
    print('all image register finished')
    return True


def face_register_img(img_path):
    """
    单张人脸图片录入接口
    :param img_path:人脸图片路径
    :return:
    """
    if not os.path.isfile(img_path):
        print('input face path is wrong, please check your path')
        return False
    filename = os.path.basename(img_path)
    img_format = filename.split('.')[-1]
    if img_format not in ['png', 'jpg', 'bmp']:
        print('{} is not a supported image format'.format(filename))
        return False
    bayes_face = BayesFaceRegister()
    user_id = filename.split('.')[0]
    res = bayes_face.register(img_path, user_id=user_id)
    if res and res['error_code'] == 0:
        print('{} register success'.format(user_id))
    elif res:
        print('{} register failed:{}'.format(user_id, res['error_msg']))
    else:
        print('发生未知异常错误')
    return True


if __name__ == '__main__':
    face_register_img('./huge.jpg')



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from abc import abstractmethod

from bayeslibs.motion import get_robot_rotate_status, robot_rotate_right, robot_rotate_left


class ApolloRotateBase:
    def __init__(self):
        pass

    @staticmethod
    def get_rotate_status():
        try:
            print('------------------------------------')
            print('转动中.....')
            rotate_stat = get_robot_rotate_status()
            while rotate_stat['status'] != 0:
                if rotate_stat['status'] == 505:
                    print('运动程序不存在')
                    return False
                rotate_stat = get_robot_rotate_status()
            print('------------------------------------')
            print('转动完成.....')
            print('------------------------------------')
            return True
        except Exception as err:
            print('发生未知错误:{}'.format(err))
            return False

    @abstractmethod
    def rotate(self, angle):
        pass


class ApolloRotateRight(ApolloRotateBase):
    def __init__(self):
        super().__init__()

    def rotate(self, angle):
        robot_rotate_right(angle)
        return self.get_rotate_status()


class ApolloRotateLeft(ApolloRotateBase):
    def __init__(self):
        super().__init__()

    def rotate(self, angle):
        robot_rotate_left(angle)
        return self.get_rotate_status()


def apollo_rotate_sample(motion_type, para):
    """
    自定义角度，让小车实现旋转运动
    :param motion_type:向左转、前右转
    :param para:
    :return:
    """
    if motion_type == '向右转':
        # 初始化Apollo转动模块实例
        apollo_move = ApolloRotateRight()
        print('------------------------------------')
        print('发送向右转{}度的运动指令'.format(para))
        apollo_move.rotate(para)  # # 向右转**度
        return True
    elif motion_type == '向左转':
        # 初始化Apollo转动模块实例
        apollo_move = ApolloRotateLeft()
        print('------------------------------------')
        print('发送向左转{}度的运动指令'.format(para))
        apollo_move.rotate(para)  # # 向左转**度
        return True
    else:
        print('------------------------------------')
        print('输入参数有误，请重新输入')
        return False


if __name__ == '__main__':
    count = 0
    # 程序运行10次，次数可修改
    # 运动指令：向右转，向左转
    # 运动参数：10,30,60,90等角度
    while count < 10:
        motion_type_ = input('请输入运动指令:')
        para_ = float(input('请输入运动参数:'))
        apollo_rotate_sample(motion_type_, para_)
        count += 1



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:speech to text
"""
from bayeslibs.utils.bridge.vbridge import asr_bridge
from bayeslibs.config import const


class ApolloASRer:
    """
    语音识别模块封装类
    """

    def __init__(self):
        pass

    @staticmethod
    def start():
        return open_asr()

    @staticmethod
    def stop():
        return close_asr()

    @staticmethod
    def status():
        return get_asr_status()


def open_asr():
    """
    开启机器人语音识别
    :param:
    :return:result
    :example:
        result = open_asr()
        ------
        result:{
            'status':0,
            'msg':'start asr success'
        }
    """
    result = asr_bridge(req_type=const.TYPE_START)
    return result


def close_asr():
    """
    关闭机器人语音识别
    :param
    :return:result
    :example:
        result = close_asr()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = asr_bridge(req_type=const.TYPE_STOP)
    return result


def get_asr_status():
    """
    查询语音识别状态
    :param
    :return:result
    :example:
        result = get_asr_status()
        ------
        result:{
            'status':0,
            'msg':'success',
            'text':'今天30度',
            'is_asr':true
        }
    """
    result = asr_bridge(req_type=const.TYPE_QUERY)
    return result



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/6/12
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.utils.bridge.mbridge import navigate_bridge
from bayeslibs.config import const, setting


class ApolloNAVer:
    """
    自动导航模块封装类，包括开始，结束，查询运动状态
    """

    def __init__(self):
        pass

    @staticmethod
    def start(destination):
        return start_auto_nav(destination)

    @staticmethod
    def stop():
        return stop_auto_nav()

    @staticmethod
    def status():
        return get_auto_nav_status()


def start_auto_nav(destination):
    """
    控制机器人导航到指定地点
    :param destination:
    :return:result
    :example:
        '''导航至destination'''
        result = start_auto_nav(destination)
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    dest = setting.ApolloConfig()
    if destination in dest.get_nav_pos_map():
        x = dest.get_nav_pos(destination)['x']
        y = dest.get_nav_pos(destination)['y']
        data = {
            'x': x,
            'y': y,
            'theta': 0
        }
        result = navigate_bridge(req_type=const.TYPE_START, data=data)
    else:
        result = {
            'status': 404,
            'msg': '{} not in map'.format(destination)
        }
    return result


def stop_auto_nav():
    """
    停止机器人的导航
    :return:result
    :example:
        result = stop_auto_nav()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = navigate_bridge(req_type=const.TYPE_STOP)
    return result


def get_auto_nav_status():
    """
    查询机器人导航状态
    :return:result
    :example:
        result = get_auto_nav_status()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = navigate_bridge(req_type=const.TYPE_QUERY)
    return result



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import open_face_detector, close_face_detector, get_faces_detected
from bayeslibs.vision import close_beauty_age_gender_recognizer
from bayeslibs.vision import get_beauty_age_genders_recognized
from bayeslibs.vision import open_beauty_age_gender_recognizer
from abc import abstractmethod
from bayeslibs.motion import robot_move_forward, get_robot_move_status, robot_move_back
import traceback
from math import pi, cos

FACE_DETECT_FACE_PROB = 0.90  # 人脸检测的概率
FACE_DISTANCE_THR = 0.5  # 人脸距离阈值

# 程序运行时间
RUN_TIME = 60
# 摄像头上斜角度
CAMERA_ANGLE = 60 * pi / 180


class ApolloMoveBase:
    def __init__(self):
        pass

    @staticmethod
    def get_move_status():
        try:
            print('------------------------------------')
            print('运动中.....')
            rotate_stat = get_robot_move_status()
            while rotate_stat['status'] != 0:
                rotate_stat = get_robot_move_status()
            print('------------------------------------')
            print('运动完成.....')
            print('------------------------------------')
            return True
        except Exception as err:
            print('发生未知错误:{}'.format(err))
            return False

    @abstractmethod
    def move(self, distance):
        pass


class ApolloMoveForward(ApolloMoveBase):
    def __init__(self):
        super().__init__()

    def move(self, distance):
        robot_move_forward(distance)
        return self.get_move_status()


class ApolloMoveBack(ApolloMoveBase):
    def __init__(self):
        super().__init__()

    def move(self, distance):
        robot_move_back(distance)
        return self.get_move_status()


class ApolloGreeter:
    __GENDER_MAP = {'male': '先生', 'female': '女士', 'Male': '先生', 'Female': '女士'}

    def __init__(self):
        self.move_forward = ApolloMoveForward()
        self.guest_gender = None

    @staticmethod
    def product_introduce(text):
        """
           输入产品讲解信息，让APOLLO机器人将其播放出来
        """
        from bayeslibs.voice import start_speak, get_speak_status
        start_res = start_speak(text)
        if start_res and start_res['status'] == 0:
            print('>>> 小贝语音合成功能打开成功 <<<')
            res = get_speak_status()
            print('------------------------------------')
            print('小贝语音合成中.......')
            while res['status'] == 0:
                res = get_speak_status()
            print('------------------------------------')
            print('>>> 小贝语音合成结束 <<<')
            print('------------------------------------')
            return True
        else:
            return False

    @staticmethod
    def dist_detect(face_pos):
        """
           利用APOLLO机器人，检测特定图像的矩形框到深度摄像头之间的距离
        """
        from bayeslibs.vision import open_distance_detector, close_distance_detector
        from bayeslibs.vision import get_distance_detected
        pos = {
            'left': int(face_pos['left']),
            'top': int(face_pos['top']),
            'width': int(face_pos['width']),
            'height': int(face_pos['height'])
        }
        open_res = open_distance_detector(pos, False)  # 打开距离检测功能
        if open_res['status'] == 0:
            print('>>> 距离检测功能打开成功 <<<')
            count = 0
            # 如果检测到对应位置的距离信息则退出程序，否则循环5次后退出
            while count < 5:
                # 获取距离检测数据，数据格式参考get_distance_detected函数说明
                dist_detected = get_distance_detected()
                # status=0表示正确检测到距离，条件判断通过后进行距离数据解析
                if dist_detected['status'] == 0:
                    print('-----------------------')
                    dist = dist_detected['data']['dist']
                    print('distance: {}m'.format(dist_detected['data']['dist']))
                    close_distance_detector()
                    return dist
                count += 1
            print('-----------------------')
            print('>>> 距离检测结束 <<<')
            print('-----------------------')
            close_distance_detector()
            return None
        else:
            return None

    @staticmethod
    def person_gender_recog(face_pos):
        """
           判断特定矩形框中人的性别
        """
        if face_pos is None:
            return None
        # 打开性别识别功能
        open_res = open_beauty_age_gender_recognizer()
        if open_res['status'] == 0:
            print('>>>性别识别功能打开成功 <<<')
            count = 0
            # 如果检测到对应位置的性别信息则退出程序，否则循环100次后退出
            while count < 500:
                # 获取性别识别数据，数据格式参考get_beauty_age_genders_recognized函数说明
                gender_recognized = get_beauty_age_genders_recognized()

                # status=0表示正确识别到性别，条件判断通过后进行性别数据解析
                if gender_recognized['status'] == 0:
                    print('识别成功，人脸数目：{}'.format(len(gender_recognized['data'])))
                    for gender in gender_recognized['data']:
                        pos_left_diff = abs(gender['left'] - face_pos['left']) < 50
                        pos_top_diff = abs(gender['top'] - face_pos['top']) < 50
                        pos_width_diff = abs(gender['width'] - face_pos['width']) < 50
                        pos_height_diff = abs(gender['height'] - face_pos['height']) < 50
                        if pos_left_diff and pos_top_diff and pos_width_diff and pos_height_diff:
                            print('-----------------------')
                            print('性别:', gender['gender'], end=', ')
                            print('年龄:', gender['age'], end=', ')
                            print('颜值:', gender['beauty'])
                            close_beauty_age_gender_recognizer()
                            print('-----------------------')
                            return gender['gender']
                count += 1
            close_beauty_age_gender_recognizer()
            return None
        else:
            return None

    def greet_run(self):
        try:
            open_res = open_face_detector()  # 打开人脸检测功能
            if open_res['status'] == 0:
                print('>>> 机器人迎宾功能打开成功 <<<')
                start_time = datetime.datetime.now()
                cur_time = datetime.datetime.now()
                while (cur_time - start_time).seconds < RUN_TIME:
                    # 获取人脸检测数据，数据格式参考get_faces_detected函数说明
                    faces = get_faces_detected()
                    # status=0表示正确检测到人脸，条件判断通过后进行人脸数据解析
                    if faces['status'] == 0:
                        print('人脸检测成功，人脸数目：{}'.format(len(faces['data'])))
                        dist = 100
                        ind = 0
                        for index, face in enumerate(faces['data']):
                            print('目标名称:', face['object_name'], end=', ')
                            print('人脸概率:', face['probability'])
                            dist_tmp = self.dist_detect(face)
                            if dist_tmp and dist_tmp < dist:
                                dist = dist_tmp
                                ind = index
                        if faces['data'][ind]['probability'] < FACE_DETECT_FACE_PROB:
                            continue
                        if dist == 100:
                            continue
                        if 0 < dist < FACE_DISTANCE_THR:
                            continue
                        elif dist > FACE_DISTANCE_THR:
                            close_face_detector()
                            self.guest_gender = self.person_gender_recog(faces['data'][ind])
                            print('dist:{}'.format(dist))
                            print('小贝向前运动')
                            print('-----------------------')
                            self.move_forward.move(4)
                            break
                    cur_time = datetime.datetime.now()
                print('-----------------------')
                if self.guest_gender is None:
                    self.product_introduce('贵宾，您好。请让我为您讲解我公司的产品信息')
                else:
                    self.product_introduce('{}，您好。请让我为您讲解我公司的产品信息'.format(
                        self.__GENDER_MAP[self.guest_gender]))
                self.product_introduce('APOLLO小车是贝叶斯智能自主研发、面向AI教学的机器人平台。硬件包含'
                                       '激光雷达、深度摄像头、运动底盘等核心设备。依托此平台，同学们能就定'
                                       '位导航、物体检测、人脸识别、语音交互等主流AI应用进行实践学习。')
                print('-----------------------')
                print('程序运行完成，退出程序')
                close_face_detector()
                return True
            else:
                return False
        except Exception as err:
            print('ApolloGreeter error:{}\n{}'.format(err, traceback.print_exc()))
            return False


if __name__ == '__main__':
    apollo_cruise = ApolloGreeter()
    apollo_cruise.greet_run()



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from .person.face.face_detect import open_face_detector, close_face_detector, get_faces_detected
from .person.face.face_recog import open_face_recognizer, close_face_recognizer, get_faces_recognized
from .person.face.face_register import face_register
from .person.age_gender.age_gender_recog import open_age_gender_recognizer
from .person.age_gender.age_gender_recog import close_age_gender_recognizer, get_age_genders_recognized
from .person.beauty.beauty_recog import open_beauty_age_gender_recognizer
from .person.beauty.beauty_recog import close_beauty_age_gender_recognizer, get_beauty_age_genders_recognized
from .person.emotion.emotion_recog import open_emotion_recognizer
from .person.emotion.emotion_recog import close_emotion_recognizer, get_emotions_recognized
from .person.handpose.handpose_recog import open_handpose_recognizer
from .person.handpose.handpose_recog import close_handpose_recognizer, get_handposes_recognized
from .person.headpose.headpose_recog import open_headpose_recognizer
from .person.headpose.headpose_recog import close_headpose_recognizer, get_headpose_recognized
from .person.skeleton.skeleton_recog import open_skeleton_recognizer
from .person.skeleton.skeleton_recog import close_skeleton_recognizer, get_skeletons_recognized
from .object.color.color_recog import open_single_color_recognizer, open_multi_color_recognizer
from .object.color.color_recog import close_color_recognizer, get_colors_recognized
from .object.distance.distance_detect import open_distance_detector, close_distance_detector, get_distance_detected
from .object.object.object_detect import open_object_detector, close_object_detector, get_objects_detected



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.config.setting import ApolloConfig
from bayeslibs.utils.comm.http import Http

APOLLO_CONFIG = ApolloConfig()


def object_detect_bridge(req_type, is_show=True):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type,
        'is_show': is_show
    }
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('vision/object_detect'), request_json)
    return res


def distance_detect_bridge(req_type, pos=None, is_show=True):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type,
        'is_show': is_show
    }
    if pos:
        request_json['pos'] = pos
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('vision/distance_detect'), request_json)
    return res


def color_recog_bridge(req_type, colors='red', is_multi=False, is_show=True):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type,
        'is_show': is_show,
        'is_multi': is_multi,
        'colors': colors
    }
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('vision/color_recog'), request_json)
    return res


def face_detect_bridge(req_type, is_show=True):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type,
        'is_show': is_show
    }
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('vision/face_detect'), request_json)
    return res


def face_recog_bridge(req_type, is_show=True):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type,
        'is_show': is_show
    }
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('vision/face_recog'), request_json)
    return res


def age_gender_recog_bridge(req_type, is_show=True):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type,
        'is_show': is_show
    }
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('vision/age_gender_recog'), request_json)
    return res


def emotion_recog_bridge(req_type, is_show=True):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type,
        'is_show': is_show
    }
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('vision/emotion_recog'), request_json)
    return res


def headpose_recog_bridge(req_type, is_show=True):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type,
        'is_show': is_show
    }
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('vision/headpose_recog'), request_json)
    return res


def beauty_recog_bridge(req_type, is_show=True):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type,
        'is_show': is_show
    }
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('vision/beauty_recog'), request_json)
    return res


def handpose_recog_bridge(req_type, is_show=True):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type,
        'is_show': is_show
    }
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('vision/handpose_recog'), request_json)
    return res


def skeleton_recog_bridge(req_type, is_show=True):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type,
        'is_show': is_show
    }
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('vision/skeleton_recog'), request_json)
    return res



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import tkinter.messagebox
import subprocess
import re
from tkinter import Tk


class BayesLibsConfig:
    def __init__(self, ip, port, user_id=None, password=None):
        self.ip = ip
        self.port = port
        self.network = None
        self.user_id = user_id
        self.password = password

    def ping_test(self):
        p_intranet = subprocess.Popen(["ping", self.ip, '-n', '5'],
                                      stdin=subprocess.PIPE,
                                      stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE,
                                      shell=True)
        outernet_ip = 'aip.baidubce.com'
        p_outernet = subprocess.Popen(["ping", outernet_ip, '-n', '5'],
                                      stdin=subprocess.PIPE,
                                      stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE,
                                      shell=True)
        out_intranet = p_intranet.stdout.read().decode('gbk')
        out_outernet = p_outernet.stdout.read().decode('gbk')
        if 'Reply' in out_intranet:
            regex_delay = re.compile("Average = (\d+)ms", re.IGNORECASE)
            regex_loss = re.compile("(\d+)% loss", re.IGNORECASE)
        else:
            regex_delay = re.compile("平均 = (\d+)ms", re.IGNORECASE)
            regex_loss = re.compile("(\d+)% 丢失", re.IGNORECASE)
        delay_intranet_list = regex_delay.findall(out_intranet)
        if not delay_intranet_list:
            root = Tk()
            root.withdraw()
            root.wm_attributes('-topmost', 1)
            tkinter.messagebox.showerror('警告', '无法连接内网，请检查设备是否在同一局域网')
            self.network = 'bad'
            return
        loss_intranet_list = regex_loss.findall(out_intranet)
        delay_outernet_list = regex_delay.findall(out_outernet)
        loss_outernet_list = regex_loss.findall(out_outernet)
        if not delay_outernet_list:
            root = Tk()
            root.withdraw()
            root.wm_attributes('-topmost', 1)
            tkinter.messagebox.showerror('错误', '无法连接外网，请检查本机网络环境')
            self.network = 'bad'
            return
        delay_intranet = int(delay_intranet_list[0])
        loss_intranet = int(loss_intranet_list[0])
        delay_outernet = int(delay_outernet_list[0])
        loss_outernet = int(loss_outernet_list[0])
        if delay_intranet > 100 or loss_intranet > 5 or delay_outernet > 100 or loss_outernet > 5:
            root = Tk()
            root.withdraw()
            root.wm_attributes('-topmost', 1)
            tkinter.messagebox.showerror('错误', '检测到网络信号差，请调整网络至Bayes网络标准')
            self.network = 'bad'
            return
        else:
            root = Tk()
            root.withdraw()
            root.wm_attributes('-topmost', 1)
            tkinter.messagebox.showinfo('提示', '网络信号达到Bayes网络标准')
            self.network = 'good'
            return

    def connect(self):
        from bayeslibs.voice.chat.small_talk import ApolloChatter
        from bayeslibs.config.setting import ApolloConfig
        apollo = ApolloConfig()
        self.ping_test()
        apollo.set_apollo_url(self.ip, self.port, self.network)
        chatter = ApolloChatter()
        res = chatter.start('连接成功')
        if res and res['status'] == 0:
            flag = True
            root = Tk()
            root.withdraw()
            root.wm_attributes('-topmost', 1)
            tkinter.messagebox.showinfo('提示', '连接成功')
            print('connect success, apollo ip:{}'.format(apollo.get_apollo_url()))
        else:
            flag = False
            root = Tk()
            root.withdraw()
            root.wm_attributes('-topmost', 1)
            tkinter.messagebox.showwarning('提示', '无法连接，请检查是否完成初始化配置')
            print('connect failed, please reset apollo\'s ip')
        return flag


def add_slam_pos(dest, x, y):
    """
        设置室内导航目标地点的坐标位置
        :param dest: 目标地点描述，英文
        :param x: RVIZ点x坐标
        :param y: RVIZ点y坐标
        :return:
    """
    from bayeslibs.config.setting import ApolloConfig
    apollo = ApolloConfig()
    apollo.set_nav_pos(dest, x, y)


def get_slam_pos():
    from bayeslibs.config.setting import ApolloConfig
    apollo = ApolloConfig()
    return apollo.get_nav_pos_map()


__all__ = ['BayesLibsConfig', 'add_slam_pos', 'get_slam_pos']



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/7/18
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.voice import stop_music, start_music, get_music_status, open_wakeup
from bayeslibs.voice import get_wakeup_status, open_asr, get_asr_status

MUSIC_LIST = ['来自天堂的魔鬼', '国歌', '钢琴曲']


def auto_speech_recognize():
    """
    APOLLO机器人语音识别
    """
    res_asr = open_asr()
    if res_asr and res_asr['status'] == 0:
        print('>>> 小贝语音识别功能打开成功 <<<')
        stat = get_asr_status()
        print('------------------------------------')
        print('小贝识别中.......')
        while stat and stat['is_asr']:
            stat = get_asr_status()
        print('------------------------------------')
        if 'text' in stat and stat['text']:
            print('小贝识别成功:{}'.format(stat['text']))
            print('------------------------------------')
            res = stat['text']
            return True, res
        else:
            res = '不好意思，小贝没有听清你的声音'
            return False, res
    else:
        return False, '网络故障'


def wakeup():
    """
    唤醒小贝
    """
    open_res = open_wakeup()
    if open_res and open_res['status'] == 0:
        print('>>> 小贝唤醒识别功能打开成功 <<<')
        res = get_wakeup_status()
        print('------------------------------------')
        print('小贝等待唤醒中.......')
        while res['status'] != 0:
            res = get_wakeup_status()
        print('------------------------------------')
        print('小贝唤醒成功，唤醒角度:{}'.format(res['angle']))
        return True
    else:
        return False


def music_play(text, play_time):
    """
    输入歌曲名称，让APOLLO机器人将其播放出来，实现音乐播放功能
    """
    start_res = start_music(text)
    if start_res and start_res['status'] == 0:
        print('>>> 小贝音乐播放功能打开成功 <<<')
        res = get_music_status()
        start_time = datetime.datetime.now()
        cur_time = datetime.datetime.now()
        print('------------------------------------')
        print('小贝音乐播放中.......')
        while (cur_time - start_time).seconds < play_time and res['status'] == 0:
            res = get_music_status()
            cur_time = datetime.datetime.now()
        print('------------------------------------')
        print('小贝音乐播放结束，退出程序')
        print('------------------------------------')
        return True
    else:
        return False


def music_play_by_vui_sample():
    """
    进行语音交互：唤醒 “小贝”，让其播放自己想听的歌曲，目前支持['来自天堂的魔鬼', '国歌', '钢琴曲']
    """
    # 首先进行语音唤醒
    w_res = wakeup()
    if not w_res:
        print('小贝唤醒失败，请监测网络设备')
        return False
    # 唤醒成功后进行语音识别
    ret, text_reg = auto_speech_recognize()
    if not ret:
        print('小贝识别失败，请重新唤醒和识别')
        return False
    for music_name in MUSIC_LIST:
        # 判断支持的歌曲是否在识别的文字里
        if music_name in text_reg:
            # 播放音乐,播放时长为30s，歌曲时长不足30s的自动停止，播放时长可调
            music_play(music_name, 30)
            # 停止播放音乐
            stop_music()
            return True
    return False


if __name__ == '__main__':
    m_res = music_play_by_vui_sample()
    while not m_res:
        print('音乐播放失败，继续识别')
        m_res = music_play_by_vui_sample()
    print('音乐播放成功，退出程序')



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import open_face_detector, get_faces_detected, close_face_detector

# 程序运行时间
RUN_TIME = 10


def multi_face_detect_sample():
    """
       多人人脸检测
    """
    open_res = open_face_detector()  # 打开人脸检测功能
    if open_res['status'] == 0:
        print('>>> 多人人脸检测功能打开成功 <<<')
        start_time = datetime.datetime.now()
        cur_time = datetime.datetime.now()
        while (cur_time - start_time).seconds < RUN_TIME:
            # 获取人脸检测数据，数据格式参考get_faces_detected函数说明
            faces_detected = get_faces_detected()
            print('-----------------------')
            # status=0表示正确检测到人脸，条件判断通过后进行人脸数据解析
            if faces_detected['status'] == 0:
                print('人脸检测成功，人脸数目：{}'.format(len(faces_detected['data'])))
                for face in faces_detected['data']:
                    print('目标名称:', face['object_name'], end=', ')
                    print('人脸概率:', face['probability'])
            else:
                print('没有检测到人脸信息，请调整至合适位置')
            cur_time = datetime.datetime.now()
        print('-----------------------')
        print('{}s运行时间已到，退出程序'.format(RUN_TIME))
        close_face_detector()
        return True
    else:
        return False


if __name__ == '__main__':
    multi_face_detect_sample()



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import open_single_color_recognizer, get_colors_recognized
from bayeslibs.vision import close_color_recognizer

COLOR_MAP = {'red': '红色', 'blue': '蓝色', 'green': '绿色', 'yellow': '黄色'}
# 程序运行时间
RUN_TIME = 10


def single_color_recog_sample():
    """
       利用单一色卡教具，观察APOLLO如何实现单一颜色识别并将返回的数据结果进行解析
       目前支持的颜色：red,blue,green,yellow
    """
    color = input('请输入你要识别的颜色：')
    while color not in COLOR_MAP.keys():
        color = input('颜色输入错误，请重新输入你要识别的颜色：')
    color = color.strip()  # 消除字符串两边的空格
    open_res = open_single_color_recognizer(color)  # 打开颜色检测功能
    if open_res and open_res['status'] == 0:
        print('>>> 单一色卡识别功能打开成功 <<<')
        print('*** 提示1：请将单一色卡教具放置在Apollo小车的RealSense摄像头可捕捉的范围内 ***')
        print('*** 提示2：程序运行时间可通过修改参数RUN_TIME实现，默认运行30s***')
        print('*** 提示3：如果程序结果提示计算机积极拒绝，说明没有配置Apollo网络，请先配置网络***')
        start_time = datetime.datetime.now()
        cur_time = datetime.datetime.now()
        while (cur_time - start_time).seconds < RUN_TIME:
            # 获取颜色检测数据，数据格式参考get_colors_recognized函数说明
            color_recognized = get_colors_recognized()
            print('------------------------------------')
            # status=0表示正确检测到颜色，条件判断通过后进行颜色数据解析
            if color_recognized and color_recognized['status'] == 0:
                color_info = color_recognized['data'][0]
                print(color_info)
                # round()将浮点数进行四舍五入变成整型数据
                print('{}色卡教具已被识别，色卡面积为{}像素'.format(COLOR_MAP[color],
                                                    round(color_info['area'])))
            else:
                print('没有识别到{}色卡教具，请调整至合适位置'.format(COLOR_MAP[color]))
            cur_time = datetime.datetime.now()
        print('------------------------------------')
        print('{}s运行时间已到，退出程序'.format(RUN_TIME))
        close_color_recognizer()
        return True
    else:
        return False


if __name__ == '__main__':
    single_color_recog_sample()



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/6/12
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.utils.bridge.ibridge import color_recog_bridge
from bayeslibs.config import const


class ApolloColorRecognizer:
    """
    颜色识别模块封装类
    """
    def __init__(self):
        pass

    @staticmethod
    def open_single(colors, is_show=True):
        return open_single_color_recognizer(colors, is_show)

    @staticmethod
    def open_multi(colors, is_show=True):
        return open_multi_color_recognizer(colors, is_show)

    @staticmethod
    def close():
        return close_color_recognizer()

    @staticmethod
    def colors():
        return get_colors_recognized()


def open_single_color_recognizer(color, is_show=True):
    """
    开启机器人单张色卡颜色识别功能
    :param
    :return:result
    :example:
        result = open_single_color_recognizer('red')
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = color_recog_bridge(req_type=const.TYPE_START, colors=color, is_multi=False, is_show=is_show)
    return result


def open_multi_color_recognizer(colors, is_show=True):
    """
    开启机器人多张色卡颜色识别功能
    :param
    :return:result
    :example:
        result = open_multi_color_recognizer('red,blue,green')
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = color_recog_bridge(req_type=const.TYPE_START, colors=colors, is_multi=True, is_show=is_show)
    return result


def close_color_recognizer():
    """
    关闭机器人颜色识别功能
    :param
    :return:result
    :example:
        result = close_color_recognizer()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = color_recog_bridge(req_type=const.TYPE_STOP)
    return result


def get_colors_recognized():
    """
    查询机器人颜色识别出的颜色信息
    :return:result
    :example:
        result = get_colors_recognized()
        ------
        result:{
            'status':0,
            'msg':'success',
            'data':[
                {
                    'color':'red',
                    'area':2130,
                    'dist':2.2,
                    'width':123,
                    'height':223,
                    'top':12,
                    'left':33
                },
                {...}
            ]
        }
    """
    result = color_recog_bridge(req_type=const.TYPE_QUERY)
    return result



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.utils.bridge.vbridge import tts_bridge
from bayeslibs.config import const


class ApolloSpeaker:
    """
    语音合成模块封装类
    """

    def __init__(self):
        pass

    @staticmethod
    def stop():
        return stop_speak()

    @staticmethod
    def speak(text):
        return start_speak(text)

    @staticmethod
    def status():
        return get_speak_status()


def start_speak(text):
    """
    机器人播放具体文本语音
    :param text
    :return:result
    :example:
        result = start_speak('今天心情很好')
        ------
        result:{
            'status':0,
            'msg':'start tts success'
        }
    """
    result = tts_bridge(req_type=const.TYPE_START, text=text)
    return result


def stop_speak():
    """
    关闭机器人语音合成
    :param
    :return:result
    :example:
        result = stop_speak()
        ------
        result:{
            'status':0,
            'msg':'stop success'
        }
    """
    result = tts_bridge(req_type=const.TYPE_STOP)
    return result


def get_speak_status():
    """
    查询机器人语音播放状态
    :param
    :return:result
    :example:
        result = get_speak_status()
        ------
        result:{
            'status':0,
            'msg':'voice is playing'
        }
    """
    result = tts_bridge(req_type=const.TYPE_QUERY)
    return result



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.utils.bridge.mbridge import move_bridge
from bayeslibs.config import const, setting

apollo_config = setting.ApolloConfig()


class ApolloMover:
    def __init__(self):
        pass

    @staticmethod
    def go_forward(distance):
        try:
            robot_move_forward(distance)
            move_stat = get_robot_move_status()
            while move_stat and move_stat['status'] != 0:
                move_stat = get_robot_move_status()
            return True
        except Exception as err:
            print('ApolloMover go_forward error:{}'.format(err))
            return False

    @staticmethod
    def go_back(distance):
        try:
            robot_move_back(distance)
            move_stat = get_robot_move_status()
            while move_stat and move_stat['status'] != 0:
                move_stat = get_robot_move_status()
            return True
        except Exception as err:
            print('ApolloMover go_back error:{}'.format(err))
            return False

    @staticmethod
    def stop_move():
        return stop_robot_move()

    @staticmethod
    def status():
        return get_robot_move_status()


def robot_move_forward(distance):
    """
    控制机器人向前运动特定距离
    :param distance:运动距离
    :return:result
    :example:
        '''向前运动3m'''
        result = robot_move_forward(3)
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    if distance > apollo_config.get_max_dist():
        distance = apollo_config.get_max_dist()
    data = {
        'x': distance,
        'y': 0,
        'theta': 0
    }
    result = move_bridge(req_type=const.TYPE_START, data=data)
    return result


def robot_move_back(distance):
    """
    控制机器人向后运动特定距离
    :param distance:运动距离
    :return:result
    :example:
        '''向后运动3m'''
        result = robot_move_back(3)
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    if distance > apollo_config.get_max_dist():
        distance = apollo_config.get_max_dist()
    data = {
        'x': -distance,
        'y': 0,
        'theta': 0
    }
    result = move_bridge(req_type=const.TYPE_START, data=data)
    return result


def stop_robot_move():
    """
    停止机器人的运动
    :return:result
    :example:
        result = stop_robot_move()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = move_bridge(req_type=const.TYPE_STOP)
    return result


def get_robot_move_status():
    """
    查询机器人状态
    :return:result
    :example:
        result = get_robot_move_status()
        ------
        result:{
            'status':0,
            'msg':'robot is moving'
        }
    """
    result = move_bridge(req_type=const.TYPE_QUERY)
    return result


if __name__ == '__main__':
    print(robot_move_forward(0.4))



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import open_headpose_recognizer, get_headpose_recognized
from bayeslibs.vision import close_headpose_recognizer

# 程序运行时间
RUN_TIME = 30


def headpose_recog_sample():
    """
       判断班级某一个人当时的头部姿态
    """
    # 打开头部姿态检测功能
    open_res = open_headpose_recognizer()
    if open_res['status'] == 0:
        print('>>> 头部姿态识别功能打开成功 <<<')
        start_time = datetime.datetime.now()
        cur_time = datetime.datetime.now()
        while (cur_time - start_time).seconds < RUN_TIME:
            # 获取头部姿态检测数据，数据格式参考get_headpose_recognized函数说明
            headposes_detected = get_headpose_recognized()
            print('-----------------------')
            # status=0表示正确检测到头部姿态，条件判断通过后进行头部姿态数据解析
            if headposes_detected['status'] == 0:
                for ind, headpose in enumerate(headposes_detected['data']):
                    print('人脸序号：{}'.format(ind))
                    print('三维旋转之左右旋转角[-90(左), 90(右)]:', headpose['yaw'])
                    print('三维旋转之俯仰角度[-90(上), 90(下)]:', headpose['pitch'])
                    print('平面内旋转角[-180(逆时针), 180(顺时针)]:', headpose['roll'])
            else:
                print('没有识别到头部姿态信息，请调整至合适位置')
            cur_time = datetime.datetime.now()
        print('-----------------------')
        print('{}s运行时间已到，退出程序'.format(RUN_TIME))
        close_headpose_recognizer()
        return True
    else:
        return False


if __name__ == '__main__':
    headpose_recog_sample()



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.utils.bridge.ibridge import face_detect_bridge
from bayeslibs.config import const


class ApolloFaceDetector:
    """
    人脸检测模块封装类
    """

    def __init__(self):
        pass

    @staticmethod
    def open(is_show=True):
        return open_face_detector(is_show)

    @staticmethod
    def close():
        return close_face_detector()

    @staticmethod
    def faces():
        return get_faces_detected()


def open_face_detector(is_show=True):
    """
    开启机器人人脸检测功能
    :param
    :return:result
    :example:
        result = open_people_detector()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = face_detect_bridge(req_type=const.TYPE_START, is_show=is_show)
    return result


def close_face_detector():
    """
    关闭机器人人脸检测功能
    :param 
    :return:result
    :example:
        result = close_people_detector()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = face_detect_bridge(req_type=const.TYPE_STOP)
    return result


def get_faces_detected():
    """
    查询机器人人脸检测出的人脸信息
    :return:result
    :example:
        result = get_faces_detected()
        ------
        result:{
            'status':0,
            'msg':'success',
            'data':[
                 {
                   'object_name':'person',
                   'probability':'1',
                   'width':123,
                   'height':223,
                   'top':12,
                   'left':33
                 },
                 {
                   'object_name':'person',
                   'probability':'1',
                   'width':53,
                   'height':63,
                   'top':122,
                   'left':133
                 },
                 ...
            ]
        }
    """
    result = face_detect_bridge(req_type=const.TYPE_QUERY)
    return result


if __name__ == '__main__':
    '''
    Python以模块运行方式启动入口
    '''
    # open_people_detector()
    close_face_detector()
    # get_faces_detected()



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import yaml
from bayeslibs.config import const
import os

HEAR = os.path.abspath(os.path.dirname(__file__))
APOLLO_INI_PATH = os.path.join(HEAR, const.APOLLO_INI)


class YamlParser:
    def __init__(self, file_path, encoding='utf-8'):
        self.file_path = file_path
        self.encoding = encoding
        self.config = self.read(file_path, encoding)

    @staticmethod
    def read(file_path, encoding):
        try:
            with open(file_path, 'r', encoding=encoding) as fp:
                config = yaml.load(fp, Loader=yaml.SafeLoader)
            return config
        except FileNotFoundError as err:
            print('file open error:{}'.format(err))
            return None

    def get(self, section, option=None):
        try:
            if not self.config:
                print('Please open a yaml file first')
                return False
            if section not in self.config.keys():
                print('section {} not in yaml file, please add first'.format(section))
                return False
            elif not option:
                return self.config[section]
            elif option not in self.config[section].keys():
                print('sub_section {} not in yaml file, please add first')
                return False
            else:
                return self.config[section][option]
        except Exception as err:
            print('Some unexpected error happen:{}'.format(err))
            return None

    def set_option(self, section, option, value):
        if not self.config:
            print('Please open a yaml file first')
            return False
        if section not in self.config.keys():
            self.config[section] = dict()
        self.config[section][option] = value
        return True

    def set(self, section, option=None, **kwargs):
        try:
            if not self.config:
                print('Please open a yaml file first')
                return False
            if section not in self.config.keys():
                self.config[section] = dict()
            if not option and kwargs:
                for item, value in kwargs.items():
                    self.config[section][item] = value
                return True
            if option and option not in self.config[section].keys():
                self.config[section][option] = dict()
            for item, value in kwargs.items():
                self.config[section][option][item] = value
            return True
        except Exception as err:
            print('Some unexpected error happen:{}'.format(err))
            return None

    def write(self):
        with open(self.file_path, 'w', encoding=self.encoding) as fw:
            yaml.dump(self.config, fw)


class ApolloConfig:

    def __init__(self, yaml_path=None):
        self.__yaml_parser = YamlParser(APOLLO_INI_PATH) if not yaml_path else YamlParser(yaml_path)
        self.__ip = self.__yaml_parser.get(const.APOLLO_T200, const.IP)
        self.__port = self.__yaml_parser.get(const.APOLLO_T200, const.PORT)
        self.__network = self.__yaml_parser.get(const.APOLLO_T200, const.NETWORK)
        self.__apollo_url = 'http://{}:{}'.format(self.__ip, self.__port)
        self.__appid = self.__yaml_parser.get(const.ACCESS_AUTH, const.APPID)
        self.__appkey = self.__yaml_parser.get(const.ACCESS_AUTH, const.APPKEY)
        self.__max_dist = self.__yaml_parser.get(const.MOTION, const.MAX_DIST)
        self.__max_angle = self.__yaml_parser.get(const.MOTION, const.MAX_ANGLE)

    def set_apollo_url(self, ip, port, network):
        self.__ip = ip
        self.__port = port
        self.__apollo_url = 'http://{}:{}'.format(ip, port)
        self.__yaml_parser.set(const.APOLLO_T200, ip=ip, port=port, network=network)
        self.save()

    def set_apollo_access(self, appid, appkey=None):
        self.__appid = appid
        self.__yaml_parser.set(const.ACCESS_AUTH, appid=appid, appkey=appkey)
        self.save()

    def set_nav_pos(self, dest, x, y):
        self.__yaml_parser.set(const.NAV_POS, option=dest, x=x, y=y)
        self.save()

    def set_common_option(self, section, option, value):
        self.__yaml_parser.set_option(section, option, value)
        self.save()

    def get_apollo_ip(self):
        return self.__ip

    def get_apollo_url(self):
        return self.__apollo_url

    def get_apollo_network(self):
        return self.__network

    def get_appid(self):
        return self.__appid

    def get_appkey(self):
        return self.__appkey

    def get_nav_pos_map(self):
        return self.__yaml_parser.get(const.NAV_POS)

    def get_nav_pos(self, dest):
        return self.__yaml_parser.get(const.NAV_POS)[dest]

    def get_robot_route_url(self, route):
        return '{}/bayes/{}'.format(self.__apollo_url, route)

    def get_max_dist(self):
        return self.__max_dist

    def get_max_angle(self):
        return self.__max_angle

    def save(self):
        self.__yaml_parser.write()



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
import time
from bayeslibs.motion import start_auto_nav, get_auto_nav_status, stop_auto_nav
from bayeslibs.config import add_slam_pos, get_slam_pos

DESTINATION_POS_MAP = {'p_hr': {'x': 6.861, 'y': 6.389}, 'p_el': {'x': -1.195, 'y': 7.257},
                       'p_origin': {'x': 0, 'y': 0}}

RUN_TIME = 60 * 10


def update_nav_pos():
    """
    更新室内导航地图位置信息
    :return: True
    """
    for item, pos in DESTINATION_POS_MAP.items():
        add_slam_pos(item, pos['x'], pos['y'])
    return True


def dest_valid(dest):
    if dest not in get_slam_pos():
        return False
    else:
        return True


def auto_nav_sample(dest):
    stat_res = start_auto_nav(dest)
    if stat_res and stat_res['status'] == 0:
        print('>>> 小贝自动导航功能打开成功 <<<')
        start_time = datetime.datetime.now()
        cur_time = datetime.datetime.now()
        stat = get_auto_nav_status()
        print('------------------------------------')
        print('小贝导航中.......')
        # 300s如果还没有导航成功就取消导航
        while stat and stat['status'] != 0 and (cur_time - start_time).seconds <= RUN_TIME:
            stat = get_auto_nav_status()
            cur_time = datetime.datetime.now()
        if (cur_time - start_time).seconds > RUN_TIME:
            print('------------------------------------')
            print('小贝导航超时.......')
            res = stop_auto_nav()
            if res['status'] == 0:
                print('导航关闭成功')
                return False
        print('------------------------------------')
        print('小贝导航成功.......')
        print('------------------------------------')
        return True, 0
    else:
        print('网络故障，请检查网络状况是否良好')
        return False, 204


if __name__ == '__main__':
    # 更新室内导航地图位置信息
    update_nav_pos()
    # 导航地点为DESTINATION_POS_MAP里的地点，输入格式为p_hr,p_el,p_origin
    valid = False
    dest_list = list()
    while not valid:
        dest_es = input('请输入导航地点:')
        dest_list = dest_es.split(',')
        valid = True
        for dest_ in dest_list:
            if not dest_valid(dest_.strip()):
                print('导航地点输入有误，请重新输入')
                valid = False
                break
    for dest_ in dest_list:
        dest_ = dest_.strip()
        # 导航到地点
        auto_nav_sample(dest_)
        # 暂停2s，再次导航到下一地点
        time.sleep(2)



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/6/12
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.utils.bridge.ibridge import object_detect_bridge
from bayeslibs.config import const


class ApolloObjectDetector:
    """
    物体检测模块封装类
    """

    def __init__(self):
        pass

    @staticmethod
    def open(is_show=True):
        return open_object_detector(is_show)

    @staticmethod
    def close():
        return close_object_detector()

    @staticmethod
    def objects():
        return get_objects_detected()


def open_object_detector(is_show=True):
    """
    开启机器人物体检测功能
    :param
    :return:result
    :example:
        result = open_object_detector()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = object_detect_bridge(req_type=const.TYPE_START, is_show=is_show)
    return result


def close_object_detector():
    """
    关闭机器人物体检测功能
    :param
    :return:result
    :example:
        result = close_object_detector()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = object_detect_bridge(req_type=const.TYPE_STOP)
    return result


def get_objects_detected():
    """
    查询机器人物体检测出的物体信息
    :return:result
    :example:
        result = get_objects_detected()
        ------
        result:{
            'status':0,
            'msg':'success',
            'data':[
                 {
                   'object_name':'person',
                   'probability':'1',
                   'width':123,
                   'height':223,
                   'top':12,
                   'left':33
                 },
                 {
                   'object_name':'bottle',
                   'probability':'1',
                   'width':53,
                   'height':63,
                   'top':122,
                   'left':133
                 },
                 ...
            ]
        }
    """
    result = object_detect_bridge(req_type=const.TYPE_QUERY)
    return result



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import time
from bayeslibs.voice import open_wakeup, get_wakeup_status, close_wakeup


def wakeup_sample(res_word):
    """
    呼唤“小贝、小贝”，观察APOLLO小车的唤醒情况
    """
    open_res = open_wakeup(res_word)
    if open_res and open_res['status'] == 0:
        print('>>> 小贝唤醒识别功能打开成功 <<<')
        res = get_wakeup_status()
        while res['status'] != 0:
            print('------------------------------------')
            res = get_wakeup_status()
            print(res)
            print('小贝等待唤醒中.......')
        print('------------------------------------')
        print('小贝唤醒成功，唤醒角度:{}'.format(res['angle']))
        print('------------------------------------')
        print('语音播放中.......')
        print('------------------------------------')
        # 暂停播放答复词，暂停时长随答复词长短而定
        time.sleep(2)
        c_res = close_wakeup()
        print(c_res)
        print('运行结束，退出程序')
        return True
    else:
        return False


if __name__ == '__main__':
    res_word_ = input('请输入唤醒小贝后的答复词:')
    wakeup_sample(res_word_)



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import open_handpose_recognizer, get_handposes_recognized
from bayeslibs.vision import close_handpose_recognizer

HANDPOSE_MAP = {'Fist': '石头', 'Two': '剪刀', 'Five': '布', 'Ok': 'OK'}
# 程序运行时间
RUN_TIME = 30


def handpose_recog_sample():
    """
       对摄像头前所捕捉到的手势（OK、剪刀、石头、布等）进行回显
    """
    open_res = open_handpose_recognizer()  # 打开手势识别功能
    if open_res['status'] == 0:
        print('>>> 手势识别功能打开成功 <<<')
        start_time = datetime.datetime.now()
        cur_time = datetime.datetime.now()
        while (cur_time - start_time).seconds < RUN_TIME:
            # 获取手势识别数据，数据格式参考get_handposes_recognized函数说明
            handpose_recognized = get_handposes_recognized()
            print('-----------------------')
            # status=0表示正确识别到手势，条件判断通过后进行手势数据解析
            if handpose_recognized['status'] == 0:
                print('手势识别成功，手势识别数目：{}'.format(len(handpose_recognized['data'])))
                for handpose in handpose_recognized['data']:
                    classname = handpose['classname']
                    if classname == 'Face':
                        continue
                    elif classname not in HANDPOSE_MAP:
                        print('手势不对，请调整至石头剪刀布或者OK')
                        continue
                    else:
                        classname = HANDPOSE_MAP[classname]
                        print('手势名称:', classname, end=', ')
                        print('手势概率:', handpose['probability'])
            else:
                print('没有识别到手势信息，请调整至合适位置')
            cur_time = datetime.datetime.now()
        print('-----------------------')
        print('{}s运行时间已到，退出程序'.format(RUN_TIME))
        close_handpose_recognizer()
        return True
    else:
        return False


if __name__ == '__main__':
    handpose_recog_sample()



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import open_emotion_recognizer, get_emotions_recognized
from bayeslibs.vision import close_emotion_recognizer

# 程序运行时间
RUN_TIME = 10

# 可识别的表情
EMOTION_MAP = {
    'anger': '愤怒', 'disgust': '厌恶', 'fear': '恐惧', 'happy': '高兴',
    'sad': '伤心', 'supprise': '惊讶', 'neutual': '无情绪'
}


def multi_face_emotion_recog_sample():
    """
        多人人脸表情识别
    """
    open_res = open_emotion_recognizer()  # 打开表情识别功能
    if open_res['status'] == 0:
        print('>>> 多人人脸表情识别功能打开成功 <<<')
        start_time = datetime.datetime.now()
        cur_time = datetime.datetime.now()
        while (cur_time - start_time).seconds < RUN_TIME:
            # 获取表情识别数据，数据格式参考get_emotions_recognized函数说明
            emotions_detected = get_emotions_recognized()
            print('-----------------------')
            # status=0表示正确检测到表情，条件判断通过后进行表情识别数据解析
            if emotions_detected['status'] == 0:
                print('人脸表情识别成功，人脸表情数目：{}'.format(len(emotions_detected['data'])))
                for emotion in emotions_detected['data']:
                    print('人脸表情类别:', EMOTION_MAP[emotion['emotion']])
            else:
                print('没有识别到人脸表情信息，请调整至合适位置')
            cur_time = datetime.datetime.now()
        print('-----------------------')
        print('{}s运行时间已到，退出程序'.format(RUN_TIME))
        close_emotion_recognizer()
        return True
    else:
        return False


if __name__ == '__main__':
    multi_face_emotion_recog_sample()



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.vision import face_register


def face_register_sample(face_path):
    """
        人脸注册程序
        face_path:人脸库路径
    """
    print('人脸注册开始......')
    res = face_register(face_path=face_path)
    if res is True:
        print('人脸注册成功!')
    else:
        print('人脸注册失败!')
    return res


if __name__ == '__main__':
    print('>>> 输入路径不能有中文 <<<')
    face_path_ = input('请输入人脸库所在路径:')
    face_register_sample(face_path_)



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/6/12
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.utils.bridge.ibridge import distance_detect_bridge
from bayeslibs.config import const


class ApolloDistanceDetector:
    """
    距离检测模块封装类
    """

    def __init__(self):
        pass

    @staticmethod
    def open(pos, is_show=True):
        return open_distance_detector(pos, is_show)

    @staticmethod
    def close():
        return close_distance_detector()

    @staticmethod
    def distance():
        return get_distance_detected()


def open_distance_detector(pos, is_show=True):
    """
    开启机器人距离检测功能
    :param
    :return:result
    :example:
        # pos:指定的距离测量区域
        pos = {
            'left': 270,
            'top': 190,
            'width': 10,
            'height': 10
        }
        result = open_distance_detector(pos)
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = distance_detect_bridge(req_type=const.TYPE_START, pos=pos, is_show=is_show)
    return result


def close_distance_detector():
    """
    关闭机器人距离检测功能
    :param
    :return:result
    :example:
        result = close_distance_detector()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = distance_detect_bridge(req_type=const.TYPE_STOP)
    return result


def get_distance_detected():
    """
    查询机器人距离检测出的距离信息
    :return:result
    :example:
        result = get_distance_detected()
        ------
        result:{
            'status':0,
            'msg':'success',
            'data': {
                'dist': 1.3,
                'dist_data': [2223,3123,44322,52322,...]
            }
        }
    """
    result = distance_detect_bridge(req_type=const.TYPE_QUERY)
    return result



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from .move.move import robot_move_forward, robot_move_back, stop_robot_move, get_robot_move_status
from .move.rotate import robot_rotate_left, robot_rotate_right, stop_robot_rotate, get_robot_rotate_status
from .navigate.auto_nav import start_auto_nav, stop_auto_nav, get_auto_nav_status



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import open_multi_color_recognizer, get_colors_recognized
from bayeslibs.vision import close_color_recognizer

COLOR_MAP = {'red': '红色', 'blue': '蓝色', 'green': '绿色', 'yellow': '黄色'}
# 程序运行时间
RUN_TIME = 30


def multi_color_recog_sample():
    """
       利用多颜色色卡教具，让APOLLO同时实现多种颜色的识别，并把识别出的颜色按照面积大小进行排序
       目前支持的颜色：red,blue,green,yellow
    """
    # 颜色输入中间以英文,隔开: red,blue
    colors = input('请输入你要识别的颜色：')
    input_check = False
    while not input_check:
        for color in colors.split(','):
            color = color.strip()  # 消除字符串两边的空格
            input_check = False if color not in COLOR_MAP.keys() else True
        if input_check:
            break
        colors = input('颜色输入错误，请重新输入你要识别的颜色：')
    open_res = open_multi_color_recognizer(colors)  # 打开多颜色识别功能
    if open_res and open_res['status'] == 0:
        print('>>> 多颜色色卡识别功能打开成功 <<<')
        start_time = datetime.datetime.now()
        cur_time = datetime.datetime.now()
        while (cur_time - start_time).seconds < RUN_TIME:
            # 获取颜色检测数据，数据格式参考get_colors_recognized函数说明
            colors_recognized = get_colors_recognized()
            # status=0表示正确检测到颜色，条件判断通过后进行颜色数据解析,并按照面积大小进行排序
            if colors_recognized and colors_recognized['status'] == 0:
                colors_info = colors_recognized['data']
                # 按照面积大小对颜色数据进行排序
                colors_info = sorted(colors_info, key=lambda item: item['area'], reverse=True)
                print('---------------------------------------------')
                for ind, color_info in enumerate(colors_info):
                    # round()将浮点数进行四舍五入变成整型数据
                    print('面积大小顺序：{}，{}色块已被识别，色卡面积为：{}'.format(
                        ind + 1, COLOR_MAP[color_info['color']], round(color_info['area'])))
            else:
                print('没有识别到多颜色色卡教具，请调整至合适位置')
            cur_time = datetime.datetime.now()
        print('---------------------------------------------')
        print('{}s运行时间已到，退出程序'.format(RUN_TIME))
        close_color_recognizer()
        return True
    else:
        return False


if __name__ == '__main__':
    multi_color_recog_sample()



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/7/18
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from abc import abstractmethod
from bayeslibs.motion import robot_move_forward, get_robot_move_status, robot_move_back
from bayeslibs.motion import robot_rotate_right, robot_rotate_left, stop_robot_rotate
from bayeslibs.motion import stop_robot_move, get_robot_rotate_status
from bayeslibs.voice import open_acr, get_acr_status, get_speak_status
from bayeslibs.voice import open_wakeup, get_wakeup_status
from bayeslibs.voice import start_speak


class ApolloMoveBase:
    def __init__(self):
        pass

    @staticmethod
    def get_move_status():
        try:
            print('------------------------------------')
            print('运动中.....')
            move_stat = get_robot_move_status()
            while move_stat['status'] != 0:
                if move_stat['status'] == 505:
                    print('运动程序不存在')
                    return False
                move_stat = get_robot_move_status()
            print('------------------------------------')
            print('运动完成.....')
            print('------------------------------------')
            return True
        except Exception as err:
            print('发生未知错误:{}'.format(err))
            return False

    @abstractmethod
    def move(self, distance):
        pass


class ApolloMoveForward(ApolloMoveBase):
    def __init__(self):
        super().__init__()

    def move(self, distance):
        robot_move_forward(distance)
        return self.get_move_status()


class ApolloMoveBack(ApolloMoveBase):
    def __init__(self):
        super().__init__()

    def move(self, distance):
        robot_move_back(distance)
        return self.get_move_status()


class ApolloRotateBase:
    def __init__(self):
        pass

    @staticmethod
    def get_rotate_status():
        try:
            print('------------------------------------')
            print('转动中.....')
            rotate_stat = get_robot_rotate_status()
            while rotate_stat['status'] != 0:
                if rotate_stat['status'] == 505:
                    print('运动程序不存在')
                    return False
                rotate_stat = get_robot_rotate_status()
            print('------------------------------------')
            print('转动完成.....')
            print('------------------------------------')
            return True
        except Exception as err:
            print('发生未知错误:{}'.format(err))
            return False

    @abstractmethod
    def rotate(self, angle):
        pass


class ApolloRotateRight(ApolloRotateBase):
    def __init__(self):
        super().__init__()

    def rotate(self, angle):
        robot_rotate_right(angle)
        return self.get_rotate_status()


class ApolloRotateLeft(ApolloRotateBase):
    def __init__(self):
        super().__init__()

    def rotate(self, angle):
        robot_rotate_left(angle)
        return self.get_rotate_status()


def wakeup():
    """
    唤醒小贝
    """
    open_res = open_wakeup()
    if open_res and open_res['status'] == 0:
        ret = get_wakeup_status()
        print('------------------------------------')
        print('小贝等待唤醒中.......')
        while ret['status'] != 0:
            ret = get_wakeup_status()
        print('------------------------------------')
        print('小贝唤醒成功，唤醒角度:{}'.format(ret['angle']))
        return True
    else:
        return False


def auto_command_recognize():
    """
    APOLLO机器人离线命令词识别
    """
    res_asr = open_acr()
    if res_asr and res_asr['status'] == 0:
        stat = get_acr_status()
        print('------------------------------------')
        print('小贝识别中.......')
        while stat and stat['is_acr']:
            stat = get_acr_status()
        print('------------------------------------')
        if 'text' in stat and stat['text']:
            print('小贝识别成功:{}'.format(stat['text']))
            print('------------------------------------')
            ret = stat['text']
            return True, ret
        else:
            ret = '不好意思，小贝没有听清你的声音'
            return False, ret
    else:
        return False, '网络故障'


def voice_play(text):
    """
       输入任意内容，让APOLLO机器人将其播放出来，实现语音合成功能
    """
    start_res = start_speak(text)
    if start_res and start_res['status'] == 0:
        res = get_speak_status()
        print('------------------------------------')
        print('小贝语音合成中.......')
        while res['status'] == 0:
            res = get_speak_status()
        return True
    else:
        return False


class CommandController:
    __DISTANCE_SUPPORT = [0.5, 1, 1.5, 2, 2.5, 3, 3.5, 4, 4.5, 5]
    __ANGLE_SUPPORT = {'十': 10, '二十': 20, '三十': 30, '四十': 40, '五十': 50,
                       '六十': 60, '七十': 70, '八十': 80, '九十': 90, '一百八十': 180,
                       '二百七十': 270, '三百六十': 360}
    __MSG_REPLY = {'forward': '好的,小贝向前走了',
                   'back': '好的,小贝往后退了',
                   'left': '好的,小贝向左转了',
                   'right': '好的,小贝向右转了',
                   'stop': '好的,小贝不走了'}

    def __init__(self):
        self.m_type = None
        self.m_value = None
        self.mover_f = ApolloMoveForward()
        self.move_b = ApolloMoveBack()
        self.rotate_l = ApolloRotateLeft()
        self.rotate_r = ApolloRotateRight()

    def m_value_attain(self, msg):
        for dist in self.__DISTANCE_SUPPORT:
            if str(dist) in msg:
                return dist
        angle_list = []
        for angle in self.__ANGLE_SUPPORT.keys():
            if angle in msg:
                angle_list.append(angle)
        if len(angle_list) == 1:
            return self.__ANGLE_SUPPORT[angle_list[0]]
        if len(angle_list) == 2:
            angle_list.remove('十')
            return self.__ANGLE_SUPPORT[angle_list[0]]
        return None

    def speak(self):
        print('------------------------------------')
        print(self.__MSG_REPLY[self.m_type])
        voice_play(self.__MSG_REPLY[self.m_type])

    def move_forward(self):
        self.m_type = 'forward'
        self.speak()
        if not self.m_value:
            self.mover_f.move(0.5)
        else:
            self.mover_f.move(self.m_value)

    def move_back(self):
        self.m_type = 'back'
        self.speak()
        if not self.m_value:
            self.move_b.move(0.5)
        else:
            self.move_b.move(self.m_value)

    def rotate_left(self):
        self.m_type = 'left'
        self.speak()
        if not self.m_value:
            self.rotate_l.rotate(90)
        else:
            print(self.m_value)
            self.rotate_l.rotate(self.m_value)

    def rotate_right(self):
        self.m_type = 'right'
        self.speak()
        if not self.m_value:
            self.rotate_r.rotate(90)
        else:
            print(self.m_value)
            self.rotate_r.rotate(self.m_value)

    def stop(self):
        self.speak()
        stop_robot_move()
        stop_robot_rotate()
        self.m_type = 'stop'

    def m_control(self, msg):
        # 提取命令词里的运动参数
        self.m_value = self.m_value_attain(msg)
        # 判断运动类型，并执行相应操作
        if '前' in msg or '前进' in msg or '向前' in msg or '往前' in msg or '前走' in msg:
            self.move_forward()
        elif '后' in msg or '后退' in msg or '向后' in msg or '往后' in msg:
            self.move_back()
        elif '向左' in msg or '往左' in msg or '左转' in msg:
            self.rotate_left()
        elif '向右' in msg or '往右' in msg or '右转' in msg:
            self.rotate_right()
        elif '停止运动' in msg or 'stop' in msg:
            self.stop()
        else:
            print('------------------------------------')
            start_speak('不好意思，小贝没有识别到正确的指令')


def motion_control_by_voice_sample():
    """
    通过语音控制，自定义距离，命令机器人向前/向后/左转/右转运动
    :return:
    """
    # 首先进行语音唤醒
    w_res = wakeup()
    if not w_res:
        print('小贝唤醒失败，请监测网络设备')
        return False
    # 唤醒成功后进行语音识别
    ret, msg = auto_command_recognize()
    if not ret:
        print('小贝识别失败，请重新唤醒和识别')
        return False
    print('------------------------------------')
    print('小贝识别成功:{}'.format(msg))
    control_ins = CommandController()
    control_ins.m_control(msg)
    return True


if __name__ == '__main__':
    count = 0
    # 程序运行10次，次数可修改
    while count < 10:
        m_res = motion_control_by_voice_sample()
        while not m_res:
            print('------------------------------------')
            print('未识别到正确指令，继续识别')
            m_res = motion_control_by_voice_sample()
        count += 1



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.config.setting import ApolloConfig
from bayeslibs.utils.comm.http import Http

APOLLO_CONFIG = ApolloConfig()


def map_bridge(req_type, data=None):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type
    }
    if data:
        request_json['data'] = data
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('map/save'), request_json)
    return res


def move_bridge(req_type, data=None):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type
    }
    if data:
        request_json['data'] = data
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('motion/move'), request_json)
    return res


def rotate_bridge(req_type, data=None):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type
    }
    if data:
        request_json['data'] = data
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('motion/rotate'), request_json)
    return res


def navigate_bridge(req_type, data=None):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type
    }
    if data:
        request_json['data'] = data
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('navigate'), request_json)
    return res



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.utils.bridge.mbridge import rotate_bridge
from bayeslibs.config import const, setting
from cmath import pi

apollo_config = setting.ApolloConfig()


class ApolloRotator:
    def __init__(self):
        pass

    @staticmethod
    def get_rotate_status():
        rotate_stat = get_robot_rotate_status()
        while rotate_stat and rotate_stat['status'] != 0:
            rotate_stat = get_robot_rotate_status()
        return True

    def rotate_right(self, angle):
        robot_rotate_right(angle)
        self.get_rotate_status()
        return True

    def rotate_left(self, angle):
        robot_rotate_left(angle)
        self.get_rotate_status()
        return True

    @staticmethod
    def stop_rotate():
        return stop_robot_rotate()


def robot_rotate_left(angle):
    """
    控制机器人向左转动特定角度
    :param angle:转动角度
    :return:result
    :example:
        '''向左转动30度'''
        result = robot_rotate_left(30)
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    if angle > apollo_config.get_max_angle():
        angle = apollo_config.get_max_angle()
    data = {
        'x': 0,
        'y': 0,
        'theta': angle / 180 * pi
    }
    result = rotate_bridge(req_type=const.TYPE_START, data=data)
    return result


def robot_rotate_right(angle):
    """
    控制机器人向右转动特定角度
    :param angle:转动角度
    :return:result
    :example:
        '''向右转动30度'''
        result = robot_rotate_right(30)
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    if angle > apollo_config.get_max_angle():
        angle = apollo_config.get_max_angle()
    data = {
        'x': 0,
        'y': 0,
        'theta': -angle / 180 * pi
    }
    result = rotate_bridge(req_type=const.TYPE_START, data=data)
    return result


def stop_robot_rotate():
    """
    停止机器人的转动
    :return:result
    :example:
        result = stop_robot_rotate()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = rotate_bridge(req_type=const.TYPE_STOP)
    return result


def get_robot_rotate_status():
    """
    查询机器人转动状态
    :return:result
    :example:
        result = get_robot_rotate_status()
        ------
        result:{
            'status':0,
            'msg':'robot is rotating'
        }
    """
    result = rotate_bridge(req_type=const.TYPE_QUERY)
    return result


if __name__ == '__main__':
    print(pi)



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/7/11
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import close_face_detector, open_single_color_recognizer
from bayeslibs.vision import get_colors_recognized, close_color_recognizer
from abc import abstractmethod
from bayeslibs.motion import robot_move_forward, get_robot_move_status
from bayeslibs.motion import robot_move_back, get_robot_rotate_status
from bayeslibs.motion import robot_rotate_right, robot_rotate_left
import traceback

COLOR_MAP = {'red': '红色', 'blue': '蓝色', 'green': '绿色', 'yellow': '黄色'}
# 程序运行时间
RUN_TIME = 60
# 色块距离阈值
COLOR_DISTANCE_THR = 0.2
IMAGE_CENTER_X = 320


class ApolloMoveBase:
    def __init__(self):
        pass

    @staticmethod
    def get_move_status():
        try:
            print('------------------------------------')
            print('运动中.....')
            rotate_stat = get_robot_move_status()
            while rotate_stat['status'] != 0:
                rotate_stat = get_robot_move_status()
            print('------------------------------------')
            print('运动完成.....')
            print('------------------------------------')
            return True
        except Exception as err:
            print('发生未知错误:{}'.format(err))
            return False

    @abstractmethod
    def move(self, distance):
        pass


class ApolloMoveForward(ApolloMoveBase):
    def __init__(self):
        super().__init__()

    def move(self, distance):
        robot_move_forward(distance)
        return self.get_move_status()


class ApolloMoveBack(ApolloMoveBase):
    def __init__(self):
        super().__init__()

    def move(self, distance):
        robot_move_back(distance)
        return self.get_move_status()


class ApolloRotateBase:
    def __init__(self):
        pass

    @staticmethod
    def get_rotate_status():
        try:
            print('------------------------------------')
            print('转动中.....')
            rotate_stat = get_robot_rotate_status()
            while rotate_stat['status'] != 0:
                if rotate_stat['status'] == 505:
                    print('运动程序不存在')
                    return False
                rotate_stat = get_robot_rotate_status()
            print('------------------------------------')
            print('转动完成.....')
            print('------------------------------------')
            return True
        except Exception as err:
            print('发生未知错误:{}'.format(err))
            return False

    @abstractmethod
    def rotate(self, angle):
        pass


class ApolloRotateRight(ApolloRotateBase):
    def __init__(self):
        super().__init__()

    def rotate(self, angle):
        robot_rotate_right(angle)
        return self.get_rotate_status()


class ApolloRotateLeft(ApolloRotateBase):
    def __init__(self):
        super().__init__()

    def rotate(self, angle):
        robot_rotate_left(angle)
        return self.get_rotate_status()


class ApolloColorTracker:
    """
       利用单一色卡教具，观察APOLLO如何实现单一颜色识别并检测色块的距离，
       不断调整与色块的距离，保证时刻追踪色块
    """

    def __init__(self):
        self.move_forward = ApolloMoveForward()
        self.rotate_right = ApolloRotateRight()
        self.rotate_left = ApolloRotateLeft()
        self.guest_gender = None

    @staticmethod
    def dist_detect(face_pos):
        """
           利用APOLLO机器人，检测特定图像的矩形框到深度摄像头之间的距离
        """
        from bayeslibs.vision import open_distance_detector, close_distance_detector
        from bayeslibs.vision import get_distance_detected
        pos = {
            'left': int(face_pos['left']),
            'top': int(face_pos['top']),
            'width': int(face_pos['width']),
            'height': int(face_pos['height'])
        }
        open_res = open_distance_detector(pos, False)  # 打开距离检测功能
        if open_res['status'] == 0:
            print('>>> 距离检测功能打开成功 <<<')
            count = 0
            # 如果检测到对应位置的距离信息则退出程序，否则循环5次后退出
            while count < 5:
                # 获取距离检测数据，数据格式参考get_distance_detected函数说明
                dist_detected = get_distance_detected()
                # status=0表示正确检测到距离，条件判断通过后进行距离数据解析
                if dist_detected['status'] == 0:
                    print('-----------------------')
                    dist = dist_detected['data']['dist']
                    print('distance: {}m'.format(dist_detected['data']['dist']))
                    close_distance_detector()
                    return dist
                count += 1
            print('-----------------------')
            print('>>> 距离检测结束 <<<')
            print('-----------------------')
            close_distance_detector()
            return None
        else:
            return None

    def track(self):
        try:
            color = input('请输入你要识别的颜色：')
            while color not in COLOR_MAP.keys():
                color = input('颜色输入错误，请重新输入你要识别的颜色：')
            color = color.strip()  # 消除字符串两边的空格
            open_res = open_single_color_recognizer(color)  # 打开颜色检测功能
            if open_res and open_res['status'] == 0:
                start_time = datetime.datetime.now()
                cur_time = datetime.datetime.now()
                print('-----------------------')
                print('{}色卡教具识别中...........'.format(COLOR_MAP[color]))
                print('-----------------------')
                while (cur_time - start_time).seconds < RUN_TIME:
                    # 获取颜色检测数据，数据格式参考get_color_recognized函数说明
                    color_recognized = get_colors_recognized()
                    # status=0表示正确检测到颜色，条件判断通过后进行颜色数据解析
                    if color_recognized and color_recognized['status'] == 0:
                        color_info = color_recognized['data'][0]
                        print('{}色卡教具已被识别'.format(COLOR_MAP[color]))
                        center_x = color_info['left'] + color_info['width'] / 2
                        dif_x = center_x - IMAGE_CENTER_X
                        print('色块中心点距离：{}'.format(dif_x))
                        if dif_x < 0 and abs(dif_x) > 30:
                            self.rotate_left.rotate(10)
                            print('-----------------------')
                            print('左转')
                            continue
                        if dif_x > 0 and abs(dif_x) > 30:
                            self.rotate_right.rotate(20)
                            print('-----------------------')
                            print('右转')
                            continue
                        dist = self.dist_detect(color_info)
                        if dist is None:
                            continue
                        print('dist:{}'.format(dist))
                        if 0 < dist < COLOR_DISTANCE_THR:
                            continue
                        elif dist > COLOR_DISTANCE_THR:
                            close_face_detector()
                            print('小贝向前运动')
                            print('-----------------------')
                            self.move_forward.move(2)
                            print('{}色卡教具识别中...........'.format(COLOR_MAP[color]))
                            print('-----------------------')
                    cur_time = datetime.datetime.now()
                print('------------------------------------')
                print('{}s运行时间已到，退出程序'.format(RUN_TIME))
                close_color_recognizer()
                return True
            else:
                return False
        except Exception as err:
            print('ApolloColorTracker error:{}\n{}'.format(err, traceback.print_exc()))
            return False


if __name__ == '__main__':
    apollo_cruise = ApolloColorTracker()
    apollo_cruise.track()



"""
@project:medical_robot_backend
@language:python3
@create:2019/4/26
@author:qianyang@aibayes.com
@description:none
"""
import json
import tkinter.messagebox
from bayeslibs.config.setting import ApolloConfig
from tkinter import Tk
from urllib.request import Request, urlopen


class Http:
    REQUEST_COUNT = 1

    def __init__(self):
        pass

    @staticmethod
    def request_json(url, data=None):
        if not data:
            return None
        if ApolloConfig().get_apollo_network() == 'bad':
            root = Tk()
            root.withdraw()
            root.wm_attributes('-topmost', 1)
            tkinter.messagebox.showerror('错误', '网络质量差，请重新配置网络')
            print('--------------------------------------------------')
            print('网络质量差，请重新配置网络')
            exit(0)
        data_rq = json.dumps(data).encode('utf-8')
        header = {'Content-Type': 'application/json'}
        req = Request(url, data_rq, header)
        try:
            f = urlopen(req, timeout=3)
            result_json = f.read().decode('utf-8')
            result_json = json.loads(result_json)
            f.close()
            Http.REQUEST_COUNT = 1
            return result_json
        except Exception as err:
            if Http.REQUEST_COUNT >= 3:
                root = Tk()
                root.withdraw()
                root.wm_attributes('-topmost', 1)
                tkinter.messagebox.showerror('错误', '请求超时，请检查初始化配置是否正确和网络信号是否良好')
                print('--------------------------------------------------')
                print('网络异常退出:{}，请检查你的网络配置是否正确，网络信号是否良好'.format(err))
                exit(0)
            else:
                Http.REQUEST_COUNT += 1
                print('Http.REQUEST_COUNT', Http.REQUEST_COUNT)
                return Http.request_json(url, data)


if __name__ == '__main__':
    pass



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.utils.bridge.ibridge import face_recog_bridge
from bayeslibs.config import const


class ApolloFaceRecognizer:
    """
    人脸识别模块封装类
    """
    def __init__(self):
        pass

    @staticmethod
    def open(is_show=True):
        return open_face_recognizer(is_show)

    @staticmethod
    def close():
        return close_face_recognizer()

    @staticmethod
    def persons():
        return get_faces_recognized()


def open_face_recognizer(is_show=True):
    """
    开启机器人人脸识别功能
    :param
    :return:result
    :example:
        result = open_face_recognizer()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = face_recog_bridge(req_type=const.TYPE_START, is_show=is_show)
    return result


def close_face_recognizer():
    """
    关闭机器人人脸识别功能
    :param 
    :return:result
    :example:
        result = close_face_recognizer()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = face_recog_bridge(req_type=const.TYPE_STOP)
    return result


def get_faces_recognized():
    """
    查询机器人人脸识别出的人脸信息
    :return:result
    :example:
        result = get_faces_recognized()
        ------
        result:{
            'status':0,
            'msg':'success',
            'data':[
                 {
                    'face_name':'qianyang',
                    'score':95,
                    'width':123,
                    'height':223,
                    'top':12,
                    'left':33
                 },
                 {
                    'face_name':'shuqing',
                    'score':99,
                    'width':63,
                    'height':53,
                    'top':121,
                    'left':133
                 },
                 {...}
            ]
        }
    """
    result = face_recog_bridge(req_type=const.TYPE_QUERY)
    return result



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.voice import start_speak, get_speak_status


def voice_play_sample(text):
    """
       输入任意内容，让APOLLO机器人将其播放出来，实现语音合成功能
    """
    start_res = start_speak(text)
    if start_res and start_res['status'] == 0:
        print('>>> 小贝语音合成功能打开成功 <<<')
        res = get_speak_status()
        while res['status'] == 0:
            print('------------------------------------')
            res = get_speak_status()
            print('小贝语音合成中.......')
        print('------------------------------------')
        print('小贝语音合成结束，退出程序')
        print('------------------------------------')
        return True
    else:
        return False


if __name__ == '__main__':
    count_ = 0
    # 程序运行10次，次数可修改
    while count_ < 10:
        question_ = input('请输入您要合成的语句:')
        print('------------------------------------')
        voice_play_sample(question_)
        count_ += 1



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import face_register
from bayeslibs.vision import open_headpose_recognizer, close_headpose_recognizer
from bayeslibs.vision import get_headpose_recognized
from bayeslibs.vision import open_emotion_recognizer, close_emotion_recognizer
from bayeslibs.vision import get_emotions_recognized

HEADPOSE_YAW_ANGLE = 30  # 人脸左右旋转阈值
HEADPOSE_PITCH_ANGLE = 40  # 人脸上下俯仰阈值
HEADPOSE_ROLL_ANGLE = 10  # 人脸平面旋转阈值
# 可识别的表情
EMOTION_MAP = {
    'anger': '愤怒', 'disgust': '厌恶', 'fear': '恐惧', 'happy': '高兴',
    'sad': '伤心', 'supprise': '惊讶', 'neutual': '无情绪'
}
RUN_TIME = 30


class ApolloStudentMonitor:
    def __init__(self):
        pass

    @staticmethod
    def voice_notice(text):
        """
           输入提示信息，让APOLLO机器人将其播放出来
        """
        from bayeslibs.voice import start_speak, get_speak_status
        start_res = start_speak(text)
        if start_res and start_res['status'] == 0:
            print('>>> 小贝语音合成功能打开成功 <<<')
            res = get_speak_status()
            print('------------------------------------')
            print('小贝语音合成中.......')
            while res['status'] == 0:
                res = get_speak_status()
            print('------------------------------------')
            print('>>> 小贝语音合成结束 <<<')
            print('------------------------------------')
            return True
        else:
            return False

    @staticmethod
    def face_register(face_path):
        """
            人脸注册程序
            face_path:人脸库路径
        """
        print('人脸注册开始......')
        res = face_register(face_path=face_path)
        if res is True:
            print('人脸注册成功!')
        else:
            print('人脸注册失败!')
        return res

    def monitor(self, file_path):
        try:
            r_res = self.face_register(file_path)
            if not r_res:
                print('人脸库录入失败，请重新录入')
                return False
            h_res = open_headpose_recognizer()
            e_res = open_emotion_recognizer(False)
            if h_res['status'] == 0 and e_res['status'] == 0:
                print('>>> 学生认真程度检查功能打开成功 <<<')
                start_time = datetime.datetime.now()
                cur_time = datetime.datetime.now()
                while (cur_time - start_time).seconds < RUN_TIME:
                    # 获取头部姿态和表情数据，数据格式参考get_headpose_recognized函数说明
                    headposes = get_headpose_recognized()
                    emotions = get_emotions_recognized()
                    # status=0表示正确检测到头部姿态，条件判断通过后进行头部姿态数据解析
                    if headposes['status'] == 0:
                        for headpose in headposes['data']:
                            yaw_flag = abs(headpose['yaw']) > HEADPOSE_YAW_ANGLE
                            pitch_flag = abs(headpose['pitch']) > HEADPOSE_PITCH_ANGLE
                            roll_flag = abs(headpose['roll']) > HEADPOSE_ROLL_ANGLE
                            if yaw_flag or pitch_flag or roll_flag:
                                self.voice_notice('同学，你的姿态不太对哦，请调整一下')
                    if emotions['status'] == 0:
                        for emotion in emotions['data']:
                            print('-----------------------')
                            print('人脸表情类别:', EMOTION_MAP[emotion['emotion']])
                    cur_time = datetime.datetime.now()
                print('-----------------------')
                print('{}s运行时间已到，退出程序'.format(RUN_TIME))
                close_headpose_recognizer()
                close_emotion_recognizer()
                return True
            else:
                return False
        except Exception as err:
            print('ApolloStudentMonitor error:{}'.format(err))
            return False


if __name__ == '__main__':
    print('>>> 输入路径不能有中文 <<<')
    face_path_ = input('请输入人脸库所在路径:')
    apollo_cruise = ApolloStudentMonitor()
    apollo_cruise.monitor(face_path_)



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.utils.bridge.ibridge import headpose_recog_bridge
from bayeslibs.config import const


class ApolloHeadPoseRecognizer:
    """
    头部姿态识别模块封装类
    """
    def __init__(self):
        pass

    @staticmethod
    def open(is_show=True):
        return open_headpose_recognizer(is_show)

    @staticmethod
    def close():
        return close_headpose_recognizer()

    @staticmethod
    def headposes():
        return get_headpose_recognized()


def open_headpose_recognizer(is_show=True):
    """
    开启机器人头部姿态识别功能
    :param
    :return:result
    :example:
        result = open_headpose_recognizer()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = headpose_recog_bridge(req_type=const.TYPE_START, is_show=is_show)
    return result


def close_headpose_recognizer():
    """
    关闭机器人头部姿态识别功能
    :param
    :return:result
    :example:
        result = close_headpose_recognizer()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = headpose_recog_bridge(req_type=const.TYPE_STOP)
    return result


def get_headpose_recognized():
    """
    查询机器人识别出的头部姿态信息
    :return:result
    :example:
        result = get_headpose_recognized()
        ------
        result:{
            'status':0,
            'msg':'success',
            'data':[
                 {
                    'yaw':-10,
                    'pitch':25,
                    'roll':23,
                    'width':63,
                    'height':223,
                    'top':12,
                    'left':33
                 },
                 ...
            ]
        }
    """
    result = headpose_recog_bridge(req_type=const.TYPE_QUERY)
    return result



"""
@project:medical_robot_backend
@language:python3
@create:2019/4/27
@author:qianyang@aibayes.com
@description:none
"""


def singleton(cls):
    _instance = {}

    def decorator(*args, **kwargs):
        if cls not in _instance:
            _instance[cls] = cls(*args, **kwargs)
        return _instance[cls]

    return decorator



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import open_beauty_age_gender_recognizer
from bayeslibs.vision import get_beauty_age_genders_recognized
from bayeslibs.vision import close_beauty_age_gender_recognizer

# 程序运行时间
RUN_TIME = 10


def beauty_age_gender_recog_sample():
    """
       判断某个人的性别以及实际年龄，并进行颜值打分
    """
    open_res = open_beauty_age_gender_recognizer()  # 打开颜值年龄性别识别功能
    if open_res['status'] == 0:
        print('>>> 颜值、年龄和性别识别功能打开成功 <<<')
        start_time = datetime.datetime.now()
        cur_time = datetime.datetime.now()
        while (cur_time - start_time).seconds < RUN_TIME:
            # 获取颜值年龄性别识别数据，数据格式参考get_beauty_age_genders_recognized函数说明
            beauty_recognized = get_beauty_age_genders_recognized()
            print('-----------------------')
            # status=0表示正确识别到颜值年龄性别，条件判断通过后进行颜值年龄性别数据解析
            if beauty_recognized['status'] == 0:
                print('识别成功，人脸数目：{}'.format(len(beauty_recognized['data'])))
                for beauty in beauty_recognized['data']:
                    print('性别:', beauty['gender'], end=', ')
                    print('年龄:', beauty['age'], end=', ')
                    print('颜值:', beauty['beauty'])
            else:
                print('没有识别到人脸信息，请调整至合适位置')
            cur_time = datetime.datetime.now()
        print('-----------------------')
        print('{}s运行时间已到，退出程序'.format(RUN_TIME))
        close_beauty_age_gender_recognizer()
        return True
    else:
        print(open_res)
        return False


if __name__ == '__main__':
    beauty_age_gender_recog_sample()



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import open_skeleton_recognizer, get_skeletons_recognized
from bayeslibs.vision import close_skeleton_recognizer

# 程序运行时间
RUN_TIME = 30


def skeleton_recog_sample():
    """
       对摄像头前所捕捉到的人体图像的骨骼进行回显
    """
    open_res = open_skeleton_recognizer()  # 打开骨骼识别功能
    if open_res['status'] == 0:
        print('>>> 骨骼识别功能打开成功 <<<')
        start_time = datetime.datetime.now()
        cur_time = datetime.datetime.now()
        while (cur_time - start_time).seconds < RUN_TIME:
            # 获取骨骼识别数据，数据格式参考get_skeletons_recognized函数说明
            skeleton_recognized = get_skeletons_recognized()
            print('-----------------------')
            # status=0表示正确检测到骨骼，条件判断通过后进行骨骼数据解析
            if skeleton_recognized['status'] == 0:
                print('人体骨骼识别成功，人体数目：{}'.format(len(skeleton_recognized['data'])))
                for skeleton_info in skeleton_recognized['data']:
                    print('骨骼关键节点:', skeleton_info['body_parts'])
            else:
                print('没有识别到骨骼信息，请调整至合适位置')
            cur_time = datetime.datetime.now()
        print('-----------------------')
        print('{}s运行时间已到，退出程序'.format(RUN_TIME))
        close_skeleton_recognizer()
        return True
    else:
        return False


if __name__ == '__main__':
    skeleton_recog_sample()



"""
@project:medical_robot_backend
@language:python3
@create:2019/4/26
@author:qianyang@aibayes.com
@description:none
"""
import pywifi

wifi = pywifi.PyWiFi()
iface = wifi.interfaces()[1]
print(iface.colors())
iface.scan()
result = iface.scan_results()
print(iface.name())

MIN_WIFI_RSSI = -100
MAX_WIFI_RSSI = -55
wifi_rssi_dict = dict()

for i in range(len(result)):
    if result[i].bssid == '9c:a6:15:9f:3d:4c':
        print(result[i].ssid, result[i].bssid, result[i].signal)
    if result[i].signal > MAX_WIFI_RSSI:
        rssi_label = 4
    elif result[i].signal < MIN_WIFI_RSSI:
        rssi_label = 0
    else:
        rssi_label = round(4 * (result[i].signal + 100) / 45)
    wifi_rssi_dict['{0}-{1}'.format(result[i].ssid, result[i].bssid)] = rssi_label
print(wifi_rssi_dict)



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from . import age_gender, emotion, headpose, face

__all__ = ['age_gender', 'emotion', 'headpose', 'face']



"""
@project:medical_robot_backend
@language:python3
@create:2019/4/26
@author:qianyang@aibayes.com
@description:none
"""
import logging
import os

_LOG_DIR = os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(__file__))), 'aiops')
if not os.path.exists(_LOG_DIR):
    os.makedirs(_LOG_DIR)
LOG_DICT = {
    'acmbot': {'log_name': os.path.join(_LOG_DIR, 'bayes_medical_acmbot.aiops'),
               'log_level': logging.DEBUG},
    'mhbot': {'log_name': os.path.join(_LOG_DIR, 'bayes_medical_bcmbot.aiops'),
              'log_level': logging.DEBUG},
    'chatbot': {'log_name': os.path.join(_LOG_DIR, 'bayes_medical_chatbot.aiops'),
                'log_level': logging.DEBUG},
    'apollo': {'log_name': os.path.join(_LOG_DIR, 'bayes_medical_chatbot.aiops'),
               'log_level': logging.DEBUG},
    'default': {'log_name': os.path.join(_LOG_DIR, 'bayes_medical_default.aiops'),
                'log_level': logging.DEBUG}
}



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from .chat.small_talk import start_small_talk, stop_small_talk
from .music.music import start_music, stop_music, get_music_status
from .speak.tts import start_speak, stop_speak, get_speak_status
from .speech.acr import open_acr, close_acr, get_acr_status
from .speech.asr import open_asr, close_asr, get_asr_status
from .wakeup.wts import open_wakeup, close_wakeup, get_wakeup_status



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/7/18
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.voice import start_small_talk, start_speak, get_speak_status


def voice_play(text):
    """
       输入任意内容，让APOLLO机器人将其播放出来，实现语音合成功能
    """
    start_res = start_speak(text)
    if start_res and start_res['status'] == 0:
        res = get_speak_status()
        print('小贝语音合成中.......')
        print('------------------------------------')
        while res['status'] == 0:
            res = get_speak_status()
        return True
    else:
        return False


class ApolloIdiom:
    """
        手动输入成语，语音播报APOLLO成语接龙的结果
    """

    def __init__(self):
        self.exit()
        self.status = False

    def idiom_input(self, idiom):
        idiom = idiom.strip()
        if self.status:
            return self.idiom_output(idiom)
        else:
            idiom = '成语接龙{}'.format(idiom)
            return self.idiom_output(idiom)

    def idiom_output(self, idiom):
        result = start_small_talk(idiom)
        if result['status'] == 0:
            answer = result['answer']
            if "退出成语接龙模式" in answer:
                self.status = False
                answer = '你接错了，请重新输入成语'
            if "进入成语接龙模式" in answer:
                self.status = True
                answer = answer.split('：')[-1]
                if self.status and answer[0] != idiom[-1]:
                    answer = '你成语写错了，我给你出个成语：{}'.format(answer)
        else:
            answer = '你接错了，请重新输入成语'
        print('小贝的回答:{}'.format(answer))
        print('------------------------------------')
        return answer

    def exit(self):
        start_small_talk('退出成语接龙')
        self.status = False


if __name__ == '__main__':
    count_ = 0
    # 初始化成语接龙实例
    apollo_idiom = ApolloIdiom()
    # 循环成语接龙10次，次数可修改
    while count_ < 10:
        question_ = input('请输入您的成语:')
        print('------------------------------------')
        answer_ = apollo_idiom.idiom_input(question_)
        voice_play(answer_)
        count_ += 1



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.config.setting import ApolloConfig
from bayeslibs.utils.comm.http import Http

APOLLO_CONFIG = ApolloConfig()


def wts_bridge(req_type, text=None):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type,
        'text': text
    }
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('voice/wts'), request_json)
    return res


def tts_bridge(req_type, text=None):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type
    }
    if text:
        request_json['text'] = text
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('voice/tts'), request_json)
    return res


def asr_bridge(req_type):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type
    }
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('voice/asr'), request_json)
    return res


def acr_bridge(req_type):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type
    }
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('voice/acr'), request_json)
    return res


def chat_bridge(req_type, text=None):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type
    }
    if text:
        request_json['text'] = text
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('voice/chat'), request_json)
    return res


def music_bridge(req_type, text=None):
    request_json = {
        'uuid': APOLLO_CONFIG.get_appid(),
        'type': req_type
    }
    if text:
        request_json['text'] = text
    res = Http.request_json(APOLLO_CONFIG.get_robot_route_url('voice/music'), request_json)
    return res



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/6/12
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.utils.bridge.vbridge import music_bridge
from bayeslibs.config import const


class ApolloMusicPlayer:
    def __init__(self, music_name):
        self.music_name = music_name

    def play(self):
        return start_music(self.music_name)

    @staticmethod
    def stop():
        return stop_music()

    @staticmethod
    def status():
        return get_music_status()


def start_music(text):
    """
    机器人播放具体音乐
    :param text
    :return:result
    :example:
        result = start_music('钢琴曲')
        ------
        result:{
            'status':0,
            'msg':'start music success'
        }
    """
    result = music_bridge(req_type=const.TYPE_START, text=text)
    return result


def stop_music():
    """
    关闭机器人音乐播放
    :param
    :return:result
    :example:
        result = stop_music()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = music_bridge(req_type=const.TYPE_STOP)
    return result


def get_music_status():
    """
    查询机器人音乐播放状态
    :param
    :return:result
    :example:
        result = get_music_status()
        ------
        result:{
            'status':0,
            'msg':'voice is playing'
        }
    """
    result = music_bridge(req_type=const.TYPE_QUERY)
    return result



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import open_object_detector, get_objects_detected, close_object_detector

# 程序运行时间
RUN_TIME = 30


def multi_object_detect_sample():
    """
       多物体检测
    """
    open_res = open_object_detector()  # 打开物体检测功能
    if open_res['status'] == 0:
        print('>>> 多物体检测功能打开成功 <<<')
        start_time = datetime.datetime.now()
        cur_time = datetime.datetime.now()
        while (cur_time - start_time).seconds < RUN_TIME:
            # 获取物体检测数据，数据格式参考get_objects_detected函数说明
            objects_detected = get_objects_detected()
            print('-----------------------')
            # status=0表示正确检测到物体，条件判断通过后进行物体数据解析
            if objects_detected['status'] == 0:
                print('物体检测成功，物体数目：{}'.format(len(objects_detected['data'])))
                for object_info in objects_detected['data']:
                    print('物体名称:', object_info['object_name'], end=', ')
                    print('物体概率:', object_info['probability'])
            else:
                print('没有检测到物体信息，请调整至合适位置')
            cur_time = datetime.datetime.now()
        print('-----------------------')
        print('{}s运行时间已到，退出程序'.format(RUN_TIME))
        close_object_detector()
        return True
    else:
        return False


if __name__ == '__main__':
    '''
    Python以模块运行方式启动入口
    '''
    multi_object_detect_sample()



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/6/27
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# 导入静态文件
file_data = [
    ("bayeslibs/config", ["bayeslibs/config/apollo.yaml"]),
]

setup(name='bayeslibs',
      version='1.0.4',
      description='The bayeslibs is the best ai education libs in the world',
      author='bayes',
      long_description=long_description,
      long_description_content_type="text/markdown",
      data_files=file_data,
      include_package_data=True,  # 是否需要导入静态数据文件
      author_email='qianyang@aibayes.com',
      install_requires=['numpy', 'PyYAML'],
      license='MIT',
      packages=find_packages(),
      zip_safe=False)



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
import datetime
from bayeslibs.vision import open_distance_detector, get_distance_detected
from bayeslibs.vision import close_distance_detector
import numpy as np

# 程序运行时间
RUN_TIME = 10


def file_path_validate(file_path):
    try:
        with open(file_path, 'a', encoding='utf-8') as f:
            f.write(str('depth data header\n'))
        return True
    except Exception as err:
        print('file save failed:{}'.format(err))
        return False


def depth_img_show_sample(file_path):
    """
       利用APOLLO机器人，捕捉摄像头前的图像，对图像的深度信息进行二维矩阵显示，并保存其深度文件
    """
    while not file_path_validate(file_path):
        file_path = input('路径输入有误，请重新输入:')
    # 限定深度图像获取范围
    pos = {
        'left': 270,
        'top': 190,
        'width': 10,
        'height': 10
    }
    # 打开距离检测功能
    open_res = open_distance_detector(pos)
    if open_res['status'] == 0:
        print('>>> 深度信息识别功能打开成功 <<<')
        start_time = datetime.datetime.now()
        cur_time = datetime.datetime.now()
        # 判断深度数据是否保存，防止文件过大
        save_flag = False
        while (cur_time - start_time).seconds < RUN_TIME:
            # 获取深度信息数据，数据格式参考get_distance_detected函数说明
            dist_detected = get_distance_detected()
            print('-----------------------------------------------------')
            # status=0表示正确检测到距离，条件判断通过后进行距离数据解析
            if dist_detected['status'] == 0:
                dist_data = np.array(dist_detected['data']['dist_data']).reshape(
                    pos['width'], pos['height'])
                print('depth data\n{}'.format(dist_data))
                if not save_flag:
                    with open(file_path, 'a', encoding='utf-8') as f:
                        f.write('{}\n'.format(str(dist_data.tolist())))
                        save_flag = True
            else:
                print('没有识别到深度信息，请调整至合适位置')
            cur_time = datetime.datetime.now()
        print('-----------------------------------------------------')
        print('{}s运行时间已到，退出程序'.format(RUN_TIME))
        close_distance_detector()
        return True
    else:
        return False


if __name__ == '__main__':
    '''
    Python以模块运行方式启动入口
    '''
    depth_save_path = input('请输入深度数据保存路径:')
    depth_img_show_sample(depth_save_path)



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/6/26
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.utils.bridge.vbridge import acr_bridge
from bayeslibs.config import const


class ApolloACRer:
    """
    语音识别模块封装类
    """

    def __init__(self):
        pass

    @staticmethod
    def open():
        return open_acr()

    @staticmethod
    def close():
        return close_acr()

    @staticmethod
    def status():
        return get_acr_status()


def open_acr():
    """
    开启机器人命令词识别
    :param:
    :return:result
    :example:
        result = open_acr()
        ------
        result:{
            'status':0,
            'msg':'start acr success'
        }
    """
    result = acr_bridge(req_type=const.TYPE_START)
    return result


def close_acr():
    """
    关闭机器人命令词识别
    :param
    :return:result
    :example:
        result = close_acr()
        ------
        result:{
            'status':0,
            'msg':'success'
        }
    """
    result = acr_bridge(req_type=const.TYPE_STOP)
    return result


def get_acr_status():
    """
    查询命令词识别状态
    :param
    :return:result
    :example:
        result = get_acr_status()
        ------
        result:{
            'status':0,
            'msg':'success',
            'text':'向前走2米',
            'is_asr':true
        }
    """
    result = acr_bridge(req_type=const.TYPE_QUERY)
    return result



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/5/30
@author:qianyang<qianyang@aibayes.com>
@description:none
"""

TYPE_START = 'start'
TYPE_STOP = 'stop'
TYPE_QUERY = 'query'

MOVE_FORWARD = 'forward'
MOVE_BACK = 'back'
MOVE_LEFT = 'left'
MOVE_RIGHT = 'right'

ROTATE_BACK = 'rotate_back'
ROTATE_LEFT = 'rotate_left'
ROTATE_RIGHT = 'rotate_right'

APOLLO_INI = 'apollo.yaml'

APOLLO_T200 = 'apollo_t200'
IP = 'ip'
PORT = 'port'
NETWORK = 'network'

ACCESS_AUTH = 'access_auth'
APPID = 'appid'
APPKEY = 'appkey'

NAV_POS = 'nav_pos'
P_ELROOM = 'p_elroom'
P_FNROOM = 'p_fnroom'
P_ORIGIN = 'p_origin'

MOTION = 'motion'
MAX_DIST = 'max_dist'
MAX_ANGLE = 'max_angle'



# conding=utf-8
"""
@project:bayeslibs
@language:Python3
@create:2019/7/10
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.voice import open_wakeup, start_speak, get_speak_status
from bayeslibs.voice import get_wakeup_status, open_asr, get_asr_status

LITERARY_STORY_LIST = ['高山流水', '庄周梦蝶', '炎黄子孙', '秦晋之好',
                       '纸上谈兵', '咏絮才高', '白云苍狗', '庖丁解牛']
LITERARY = 'LITERARY-ALLUSION'


def auto_speech_recognize():
    """
    APOLLO机器人语音识别
    """
    res_asr = open_asr()
    if res_asr and res_asr['status'] == 0:
        print('>>> 小贝语音识别功能打开成功 <<<')
        stat = get_asr_status()
        print('------------------------------------')
        print('小贝识别中.......')
        while stat and stat['is_asr']:
            stat = get_asr_status()
        print('------------------------------------')
        if 'text' in stat and stat['text']:
            print('小贝识别成功:{}'.format(stat['text']))
            print('------------------------------------')
            res = stat['text']
            return True, res
        else:
            res = '不好意思，小贝没有听清你的声音'
            return False, res
    else:
        return False, '网络故障'


def wakeup():
    """
    唤醒小贝
    """
    open_res = open_wakeup()
    if open_res and open_res['status'] == 0:
        print('>>> 小贝唤醒识别功能打开成功 <<<')
        res = get_wakeup_status()
        print('------------------------------------')
        print('小贝等待唤醒中.......')
        while res['status'] != 0:
            res = get_wakeup_status()
        print('------------------------------------')
        print('小贝唤醒成功，唤醒角度:{}'.format(res['angle']))
        return True
    else:
        return False


def voice_play(text):
    """
       输入任意内容，让APOLLO机器人将其播放出来，实现语音合成功能
    """
    print(text)
    start_res = start_speak(text)
    if start_res and start_res['status'] == 0:
        res = get_speak_status()
        print('------------------------------------')
        print('小贝语音合成中.......')
        while res['status'] == 0:
            res = get_speak_status()
        return True
    else:
        return False


def literary_story_sample():
    """
    进行语音交互：唤醒 “小贝”，让其播放自己讲解的文学典故
    目前支持的典故有：高山流水, 庄周梦蝶, 炎黄子孙, 秦晋之好,
                   纸上谈兵, 咏絮才高, 白云苍狗, 庖丁解牛
    """
    # 首先进行语音唤醒
    w_res = wakeup()
    if not w_res:
        print('小贝唤醒失败，请监测网络设备')
        return False
    # 唤醒成功后进行语音识别
    ret, text_reg = auto_speech_recognize()
    if not ret:
        print('小贝识别失败，请重新唤醒和识别')
        return False
    for music_name in LITERARY_STORY_LIST:
        # 判断支持的文学典故是否在识别的文字里
        if music_name in text_reg:
            # 讲解文学典故，直至文学典故讲解完毕
            voice_play(LITERARY + music_name)
            return True
    return False


if __name__ == '__main__':
    m_res = literary_story_sample()
    while not m_res:
        print('文学典故讲解失败，继续识别')
        m_res = literary_story_sample()
    print('文学典故讲解成功，退出程序')
